@@ -37,11 +37,11 @@ type Deps struct {
 	// Used to log warnings that may repeat itself many times.
 	DistinctWarningLog *helpers.DistinctLogger
 
-	// The templates to use. This will usually implement the full tpl.TemplateHandler.
-	Tmpl tpl.TemplateFinder `json:"-"`
+	// The templates to use. This will usually implement the full tpl.TemplateManager.
+	tmpl tpl.TemplateHandler
 
 	// We use this to parse and execute ad-hoc text templates.
-	TextTmpl tpl.TemplateParseFinder `json:"-"`
+	textTmpl tpl.TemplateParseFinder
 
 	// The file systems to use.
 	Fs *hugofs.Fs `json:"-"`
@@ -77,7 +77,10 @@ type Deps struct {
 	OutputFormatsConfig output.Formats
 
 	templateProvider ResourceProvider
-	WithTemplate     func(templ tpl.TemplateHandler) error `json:"-"`
+	WithTemplate     func(templ tpl.TemplateManager) error `json:"-"`
+
+	// Used in tests
+	OverloadedTemplateFuncs map[string]interface{}
 
 	translationProvider ResourceProvider
 
@@ -89,6 +92,9 @@ type Deps struct {
 	// BuildStartListeners will be notified before a build starts.
 	BuildStartListeners *Listeners
 
+	// Atomic flags set during a build.
+	BuildFlags *BuildFlags
+
 	*globalErrHandler
 }
 
@@ -150,9 +156,20 @@ type ResourceProvider interface {
 	Clone(deps *Deps) error
 }
 
-// TemplateHandler returns the used tpl.TemplateFinder as tpl.TemplateHandler.
-func (d *Deps) TemplateHandler() tpl.TemplateHandler {
-	return d.Tmpl.(tpl.TemplateHandler)
+func (d *Deps) Tmpl() tpl.TemplateHandler {
+	return d.tmpl
+}
+
+func (d *Deps) TextTmpl() tpl.TemplateParseFinder {
+	return d.textTmpl
+}
+
+func (d *Deps) SetTmpl(tmpl tpl.TemplateHandler) {
+	d.tmpl = tmpl
+}
+
+func (d *Deps) SetTextTmpl(tmpl tpl.TemplateParseFinder) {
+	d.textTmpl = tmpl
 }
 
 // LoadResources loads translations and templates.
@@ -218,7 +235,9 @@ func New(cfg DepsCfg) (*Deps, error) {
 		return nil, errors.WithMessage(err, "failed to create file caches from configuration")
 	}
 
-	resourceSpec, err := resources.NewSpec(ps, fileCaches, logger, cfg.OutputFormats, cfg.MediaTypes)
+	errorHandler := &globalErrHandler{}
+
+	resourceSpec, err := resources.NewSpec(ps, fileCaches, logger, errorHandler, cfg.OutputFormats, cfg.MediaTypes)
 	if err != nil {
 		return nil, err
 	}
@@ -239,24 +258,26 @@ func New(cfg DepsCfg) (*Deps, error) {
 	distinctWarnLogger := helpers.NewDistinctLogger(logger.WARN)
 
 	d := &Deps{
-		Fs:                  fs,
-		Log:                 logger,
-		DistinctErrorLog:    distinctErrorLogger,
-		DistinctWarningLog:  distinctWarnLogger,
-		templateProvider:    cfg.TemplateProvider,
-		translationProvider: cfg.TranslationProvider,
-		WithTemplate:        cfg.WithTemplate,
-		PathSpec:            ps,
-		ContentSpec:         contentSpec,
-		SourceSpec:          sp,
-		ResourceSpec:        resourceSpec,
-		Cfg:                 cfg.Language,
-		Language:            cfg.Language,
-		Site:                cfg.Site,
-		FileCaches:          fileCaches,
-		BuildStartListeners: &Listeners{},
-		Timeout:             time.Duration(timeoutms) * time.Millisecond,
-		globalErrHandler:    &globalErrHandler{},
+		Fs:                      fs,
+		Log:                     logger,
+		DistinctErrorLog:        distinctErrorLogger,
+		DistinctWarningLog:      distinctWarnLogger,
+		templateProvider:        cfg.TemplateProvider,
+		translationProvider:     cfg.TranslationProvider,
+		WithTemplate:            cfg.WithTemplate,
+		OverloadedTemplateFuncs: cfg.OverloadedTemplateFuncs,
+		PathSpec:                ps,
+		ContentSpec:             contentSpec,
+		SourceSpec:              sp,
+		ResourceSpec:            resourceSpec,
+		Cfg:                     cfg.Language,
+		Language:                cfg.Language,
+		Site:                    cfg.Site,
+		FileCaches:              fileCaches,
+		BuildStartListeners:     &Listeners{},
+		BuildFlags:              &BuildFlags{},
+		Timeout:                 time.Duration(timeoutms) * time.Millisecond,
+		globalErrHandler:        errorHandler,
 	}
 
 	if cfg.Cfg.GetBool("templateMetrics") {
@@ -287,7 +308,7 @@ func (d Deps) ForLanguage(cfg DepsCfg, onCreated func(d *Deps) error) (*Deps, er
 	// The resource cache is global so reuse.
 	// TODO(bep) clean up these inits.
 	resourceCache := d.ResourceSpec.ResourceCache
-	d.ResourceSpec, err = resources.NewSpec(d.PathSpec, d.ResourceSpec.FileCaches, d.Log, cfg.OutputFormats, cfg.MediaTypes)
+	d.ResourceSpec, err = resources.NewSpec(d.PathSpec, d.ResourceSpec.FileCaches, d.Log, d.globalErrHandler, cfg.OutputFormats, cfg.MediaTypes)
 	if err != nil {
 		return nil, err
 	}
@@ -344,7 +365,9 @@ type DepsCfg struct {
 
 	// Template handling.
 	TemplateProvider ResourceProvider
-	WithTemplate     func(templ tpl.TemplateHandler) error
+	WithTemplate     func(templ tpl.TemplateManager) error
+	// Used in tests
+	OverloadedTemplateFuncs map[string]interface{}
 
 	// i18n handling.
 	TranslationProvider ResourceProvider
@@ -352,3 +375,11 @@ type DepsCfg struct {
 	// Whether we are in running (server) mode
 	Running bool
 }
+
+// BuildFlags are flags that may be turned on during a build.
+type BuildFlags struct {
+}
+
+func NewBuildFlags() BuildFlags {
+	return BuildFlags{}
+}
