@@ -14,17 +14,20 @@
 package hugolib
 
 import (
+	"bytes"
+	"fmt"
+	"path/filepath"
 	"testing"
 
+	qt "github.com/frankban/quicktest"
 	"github.com/spf13/afero"
 	"github.com/spf13/viper"
-	"github.com/stretchr/testify/require"
 )
 
 func TestLoadConfig(t *testing.T) {
 	t.Parallel()
 
-	assert := require.New(t)
+	c := qt.New(t)
 
 	// Add a random config variable for testing.
 	// side = page in Norwegian.
@@ -37,19 +40,16 @@ func TestLoadConfig(t *testing.T) {
 	writeToFs(t, mm, "hugo.toml", configContent)
 
 	cfg, _, err := LoadConfig(ConfigSourceDescriptor{Fs: mm, Filename: "hugo.toml"})
-	require.NoError(t, err)
+	c.Assert(err, qt.IsNil)
+
+	c.Assert(cfg.GetString("paginatePath"), qt.Equals, "side")
 
-	assert.Equal("side", cfg.GetString("paginatePath"))
-	// default
-	assert.Equal("layouts", cfg.GetString("layoutDir"))
-	// no themes
-	assert.False(cfg.IsSet("allThemes"))
 }
 
 func TestLoadMultiConfig(t *testing.T) {
 	t.Parallel()
 
-	assert := require.New(t)
+	c := qt.New(t)
 
 	// Add a random config variable for testing.
 	// side = page in Norwegian.
@@ -67,16 +67,16 @@ func TestLoadMultiConfig(t *testing.T) {
 	writeToFs(t, mm, "override.toml", configContentSub)
 
 	cfg, _, err := LoadConfig(ConfigSourceDescriptor{Fs: mm, Filename: "base.toml,override.toml"})
-	require.NoError(t, err)
+	c.Assert(err, qt.IsNil)
 
-	assert.Equal("top", cfg.GetString("paginatePath"))
-	assert.Equal("same", cfg.GetString("DontChange"))
+	c.Assert(cfg.GetString("paginatePath"), qt.Equals, "top")
+	c.Assert(cfg.GetString("DontChange"), qt.Equals, "same")
 }
 
 func TestLoadConfigFromTheme(t *testing.T) {
 	t.Parallel()
 
-	assert := require.New(t)
+	c := qt.New(t)
 
 	mainConfigBasic := `
 theme = "test-theme"
@@ -188,11 +188,6 @@ map[string]interface {}{
   "p1": "p1 main",
   "p2": "p2 main",
   "p3": "p3 theme",
-  "test-theme": map[string]interface {}{
-    "p1": "p1 theme",
-    "p2": "p2 theme",
-    "p3": "p3 theme",
-  },
   "top": "top",
 }`, got["params"])
 
@@ -247,8 +242,8 @@ map[string]interface {}{
 	b.AssertObject(`map[string]interface {}{
   "en": map[string]interface {}{
     "languagename": "English",
-    "menu": map[string]interface {}{
-      "theme": []interface {}{
+    "menus": map[string]interface {}{
+      "theme": []map[string]interface {}{
         map[string]interface {}{
           "name": "menu-lang-en-theme",
         },
@@ -257,16 +252,12 @@ map[string]interface {}{
     "params": map[string]interface {}{
       "pl1": "p1-en-main",
       "pl2": "p2-en-theme",
-      "test-theme": map[string]interface {}{
-        "pl1": "p1-en-theme",
-        "pl2": "p2-en-theme",
-      },
     },
   },
   "nb": map[string]interface {}{
     "languagename": "Norsk",
-    "menu": map[string]interface {}{
-      "theme": []interface {}{
+    "menus": map[string]interface {}{
+      "theme": []map[string]interface {}{
         map[string]interface {}{
           "name": "menu-lang-nb-theme",
         },
@@ -275,11 +266,6 @@ map[string]interface {}{
     "params": map[string]interface {}{
       "pl1": "p1-nb-main",
       "pl2": "p2-nb-theme",
-      "test-theme": map[string]interface {}{
-        "pl1": "p1-nb-theme",
-        "pl2": "p2-nb-theme",
-        "top": "top-nb-theme",
-      },
     },
   },
 }
@@ -287,25 +273,25 @@ map[string]interface {}{
 
 	b.AssertObject(`
 map[string]interface {}{
-  "main": []interface {}{
+  "main": []map[string]interface {}{
     map[string]interface {}{
       "name": "menu-main-main",
     },
   },
-  "thememenu": []interface {}{
+  "thememenu": []map[string]interface {}{
     map[string]interface {}{
       "name": "menu-theme",
     },
   },
-  "top": []interface {}{
+  "top": []map[string]interface {}{
     map[string]interface {}{
       "name": "menu-top-main",
     },
   },
 }
-`, got["menu"])
+`, got["menus"])
 
-	assert.Equal("https://example.com/", got["baseurl"])
+	c.Assert(got["baseurl"], qt.Equals, "https://example.com/")
 
 	if true {
 		return
@@ -328,7 +314,7 @@ map[string]interface {}{
   },
 }`, got["params"])
 
-	assert.Nil(got["languages"])
+	c.Assert(got["languages"], qt.IsNil)
 	b.AssertObject(`
 map[string]interface {}{
   "text/m1": map[string]interface {}{
@@ -379,7 +365,7 @@ map[string]interface {}{
 func TestPrivacyConfig(t *testing.T) {
 	t.Parallel()
 
-	assert := require.New(t)
+	c := qt.New(t)
 
 	tomlConfig := `
 
@@ -394,6 +380,147 @@ privacyEnhanced = true
 	b.WithConfigFile("toml", tomlConfig)
 	b.Build(BuildCfg{SkipRender: true})
 
-	assert.True(b.H.Sites[0].Info.Config().Privacy.YouTube.PrivacyEnhanced)
+	c.Assert(b.H.Sites[0].Info.Config().Privacy.YouTube.PrivacyEnhanced, qt.Equals, true)
+
+}
+
+func TestLoadConfigModules(t *testing.T) {
+	t.Parallel()
+
+	c := qt.New(t)
+
+	// https://github.com/gohugoio/hugoThemes#themetoml
+
+	const (
+		// Before Hugo 0.56 each theme/component could have its own theme.toml
+		// with some settings, mostly used on the Hugo themes site.
+		// To preserve combability we read these files into the new "modules"
+		// section in config.toml.
+		o1t = `
+name = "Component o1"
+license = "MIT"
+min_version = 0.38
+`
+		// This is the component's config.toml, using the old theme syntax.
+		o1c = `
+theme = ["n2"]
+`
+
+		n1 = `
+title = "Component n1"
+
+[module]
+description = "Component n1 description"
+[module.hugoVersion]
+min = "0.40.0"
+max = "0.50.0"
+extended = true
+[[module.imports]]
+path="o1"
+[[module.imports]]
+path="n3"
+
+
+`
+
+		n2 = `
+title = "Component n2"
+`
+
+		n3 = `
+title = "Component n3"
+`
+
+		n4 = `
+title = "Component n4"
+`
+	)
+
+	b := newTestSitesBuilder(t)
+
+	writeThemeFiles := func(name, configTOML, themeTOML string) {
+		b.WithSourceFile(filepath.Join("themes", name, "data", "module.toml"), fmt.Sprintf("name=%q", name))
+		if configTOML != "" {
+			b.WithSourceFile(filepath.Join("themes", name, "config.toml"), configTOML)
+		}
+		if themeTOML != "" {
+			b.WithSourceFile(filepath.Join("themes", name, "theme.toml"), themeTOML)
+		}
+	}
+
+	writeThemeFiles("n1", n1, "")
+	writeThemeFiles("n2", n2, "")
+	writeThemeFiles("n3", n3, "")
+	writeThemeFiles("n4", n4, "")
+	writeThemeFiles("o1", o1c, o1t)
+
+	b.WithConfigFile("toml", `
+[module]
+[[module.imports]]
+path="n1"
+[[module.imports]]
+path="n4"
+
+`)
+
+	b.Build(BuildCfg{})
+
+	modulesClient := b.H.Paths.ModulesClient
+	var graphb bytes.Buffer
+	modulesClient.Graph(&graphb)
+
+	expected := `project n1
+n1 o1
+o1 n2
+n1 n3
+project n4
+`
+
+	c.Assert(graphb.String(), qt.Equals, expected)
+
+}
+
+func TestLoadConfigWithOsEnvOverrides(t *testing.T) {
+
+	c := qt.New(t)
+
+	baseConfig := `
+
+environment = "production"
+enableGitInfo = true
+intSlice = [5,7,9]
+floatSlice = [3.14, 5.19]
+stringSlice = ["a", "b"]
+
+[imaging]
+anchor = "smart"
+quality = 75 
+resamplefilter = "CatmullRom"
+`
+
+	b := newTestSitesBuilder(t).WithConfigFile("toml", baseConfig)
+
+	b.WithEnviron(
+		"HUGO_ENVIRONMENT", "test",
+		"HUGO_NEW", "new", // key not in config.toml
+		"HUGO_ENABLEGITINFO", "false",
+		"HUGO_IMAGING_ANCHOR", "top",
+		"HUGO_STRINGSLICE", `["c", "d"]`,
+		"HUGO_INTSLICE", `[5, 8, 9]`,
+		"HUGO_FLOATSLICE", `[5.32]`,
+	)
+
+	b.Build(BuildCfg{})
+
+	cfg := b.H.Cfg
+
+	c.Assert(cfg.Get("environment"), qt.Equals, "test")
+	c.Assert(cfg.GetBool("enablegitinfo"), qt.Equals, false)
+	c.Assert(cfg.Get("new"), qt.Equals, "new")
+	c.Assert(cfg.Get("imaging.anchor"), qt.Equals, "top")
+	c.Assert(cfg.Get("imaging.quality"), qt.Equals, int64(75))
+	c.Assert(cfg.Get("stringSlice"), qt.DeepEquals, []interface{}{"c", "d"})
+	c.Assert(cfg.Get("floatSlice"), qt.DeepEquals, []interface{}{5.32})
+	c.Assert(cfg.Get("intSlice"), qt.DeepEquals, []interface{}{5, 8, 9})
 
 }
