@@ -18,6 +18,7 @@ package filesystems
 import (
 	"io"
 	"os"
+	"path"
 	"path/filepath"
 	"strings"
 	"sync"
@@ -55,6 +56,8 @@ type BaseFs struct {
 	theBigFs *filesystemsCollector
 }
 
+// TODO(bep) we can get regular files in here and that is fine, but
+// we need to clean up the naming.
 func (fs *BaseFs) WatchDirs() []hugofs.FileMetaInfo {
 	var dirs []hugofs.FileMetaInfo
 	for _, dir := range fs.AllDirs() {
@@ -62,7 +65,6 @@ func (fs *BaseFs) WatchDirs() []hugofs.FileMetaInfo {
 			dirs = append(dirs, dir)
 		}
 	}
-
 	return dirs
 }
 
@@ -90,7 +92,7 @@ func (b *BaseFs) RelContentDir(filename string) string {
 	for _, dir := range b.SourceFilesystems.Content.Dirs {
 		dirname := dir.Meta().Filename()
 		if strings.HasPrefix(filename, dirname) {
-			rel := strings.TrimPrefix(filename, dirname)
+			rel := path.Join(dir.Meta().Path(), strings.TrimPrefix(filename, dirname))
 			return strings.TrimPrefix(rel, filePathSeparator)
 		}
 	}
@@ -126,10 +128,28 @@ type SourceFilesystems struct {
 	StaticDirs []hugofs.FileMetaInfo
 }
 
+// FileSystems returns the FileSystems relevant for the change detection
+// in server mode.
+// Note: This does currently not return any static fs.
+func (s *SourceFilesystems) FileSystems() []*SourceFilesystem {
+	return []*SourceFilesystem{
+		s.Content,
+		s.Data,
+		s.I18n,
+		s.Layouts,
+		s.Archetypes,
+		// TODO(bep) static
+	}
+
+}
+
 // A SourceFilesystem holds the filesystem for a given source type in Hugo (data,
 // i18n, layouts, static) and additional metadata to be able to use that filesystem
 // in server mode.
 type SourceFilesystem struct {
+	// Name matches one in files.ComponentFolders
+	Name string
+
 	// This is a virtual composite filesystem. It expects path relative to a context.
 	Fs afero.Fs
 
@@ -238,6 +258,7 @@ func (s SourceFilesystems) MakeStaticPathRelative(filename string) string {
 // MakePathRelative creates a relative path from the given filename.
 // It will return an empty string if the filename is not a member of this filesystem.
 func (d *SourceFilesystem) MakePathRelative(filename string) string {
+
 	for _, dir := range d.Dirs {
 		meta := dir.(hugofs.FileMetaInfo).Meta()
 		currentPath := meta.Filename()
@@ -275,6 +296,22 @@ func (d *SourceFilesystem) Contains(filename string) bool {
 	return false
 }
 
+// Path returns the mount relative path to the given filename if it is a member of
+// of the current filesystem, an empty string if not.
+func (d *SourceFilesystem) Path(filename string) string {
+	for _, dir := range d.Dirs {
+		meta := dir.Meta()
+		if strings.HasPrefix(filename, meta.Filename()) {
+			p := strings.TrimPrefix(strings.TrimPrefix(filename, meta.Filename()), filePathSeparator)
+			if mountRoot := meta.MountRoot(); mountRoot != "" {
+				return filepath.Join(mountRoot, p)
+			}
+			return p
+		}
+	}
+	return ""
+}
+
 // RealDirs gets a list of absolute paths to directories starting from the given
 // path.
 func (d *SourceFilesystem) RealDirs(from string) []string {
@@ -349,12 +386,14 @@ func newSourceFilesystemsBuilder(p *paths.Paths, logger *loggers.Logger, b *Base
 	return &sourceFilesystemsBuilder{p: p, logger: logger, sourceFs: sourceFs, theBigFs: b.theBigFs, result: &SourceFilesystems{}}
 }
 
-func (b *sourceFilesystemsBuilder) newSourceFilesystem(fs afero.Fs, dirs []hugofs.FileMetaInfo) *SourceFilesystem {
+func (b *sourceFilesystemsBuilder) newSourceFilesystem(name string, fs afero.Fs, dirs []hugofs.FileMetaInfo) *SourceFilesystem {
 	return &SourceFilesystem{
+		Name: name,
 		Fs:   fs,
 		Dirs: dirs,
 	}
 }
+
 func (b *sourceFilesystemsBuilder) Build() (*SourceFilesystems, error) {
 
 	if b.theBigFs == nil {
@@ -369,12 +408,12 @@ func (b *sourceFilesystemsBuilder) Build() (*SourceFilesystems, error) {
 
 	createView := func(componentID string) *SourceFilesystem {
 		if b.theBigFs == nil || b.theBigFs.overlayMounts == nil {
-			return b.newSourceFilesystem(hugofs.NoOpFs, nil)
+			return b.newSourceFilesystem(componentID, hugofs.NoOpFs, nil)
 		}
 
 		dirs := b.theBigFs.overlayDirs[componentID]
 
-		return b.newSourceFilesystem(afero.NewBasePathFs(b.theBigFs.overlayMounts, componentID), dirs)
+		return b.newSourceFilesystem(componentID, afero.NewBasePathFs(b.theBigFs.overlayMounts, componentID), dirs)
 
 	}
 
@@ -392,14 +431,14 @@ func (b *sourceFilesystemsBuilder) Build() (*SourceFilesystems, error) {
 		return nil, err
 	}
 
-	b.result.Data = b.newSourceFilesystem(dataFs, dataDirs)
+	b.result.Data = b.newSourceFilesystem(files.ComponentFolderData, dataFs, dataDirs)
 
 	i18nDirs := b.theBigFs.overlayDirs[files.ComponentFolderI18n]
 	i18nFs, err := hugofs.NewSliceFs(i18nDirs...)
 	if err != nil {
 		return nil, err
 	}
-	b.result.I18n = b.newSourceFilesystem(i18nFs, i18nDirs)
+	b.result.I18n = b.newSourceFilesystem(files.ComponentFolderI18n, i18nFs, i18nDirs)
 
 	contentDirs := b.theBigFs.overlayDirs[files.ComponentFolderContent]
 	contentBfs := afero.NewBasePathFs(b.theBigFs.overlayMountsContent, files.ComponentFolderContent)
@@ -409,7 +448,7 @@ func (b *sourceFilesystemsBuilder) Build() (*SourceFilesystems, error) {
 		return nil, errors.Wrap(err, "create content filesystem")
 	}
 
-	b.result.Content = b.newSourceFilesystem(contentFs, contentDirs)
+	b.result.Content = b.newSourceFilesystem(files.ComponentFolderContent, contentFs, contentDirs)
 
 	b.result.Work = afero.NewReadOnlyFs(b.theBigFs.overlayFull)
 
@@ -421,13 +460,13 @@ func (b *sourceFilesystemsBuilder) Build() (*SourceFilesystems, error) {
 	if b.theBigFs.staticPerLanguage != nil {
 		// Multihost mode
 		for k, v := range b.theBigFs.staticPerLanguage {
-			sfs := b.newSourceFilesystem(v, b.result.StaticDirs)
+			sfs := b.newSourceFilesystem(files.ComponentFolderStatic, v, b.result.StaticDirs)
 			sfs.PublishFolder = k
 			ms[k] = sfs
 		}
 	} else {
 		bfs := afero.NewBasePathFs(b.theBigFs.overlayMountsStatic, files.ComponentFolderStatic)
-		ms[""] = b.newSourceFilesystem(bfs, b.result.StaticDirs)
+		ms[""] = b.newSourceFilesystem(files.ComponentFolderStatic, bfs, b.result.StaticDirs)
 	}
 
 	return b.result, nil
@@ -497,8 +536,11 @@ func (b *sourceFilesystemsBuilder) createModFs(
 		fromToStatic  []hugofs.RootMapping
 	)
 
-	absPathify := func(path string) string {
-		return paths.AbsPathify(md.dir, path)
+	absPathify := func(path string) (string, string) {
+		if filepath.IsAbs(path) {
+			return "", path
+		}
+		return md.dir, paths.AbsPathify(md.dir, path)
 	}
 
 	for _, mount := range md.Mounts() {
@@ -508,9 +550,13 @@ func (b *sourceFilesystemsBuilder) createModFs(
 			mountWeight++
 		}
 
+		base, filename := absPathify(mount.Source)
+
 		rm := hugofs.RootMapping{
-			From: mount.Target,
-			To:   absPathify(mount.Source),
+			From:      mount.Target,
+			To:        filename,
+			ToBasedir: base,
+			Module:    md.Module.Path(),
 			Meta: hugofs.FileMeta{
 				"watch":       md.Watch(),
 				"mountWeight": mountWeight,
@@ -585,7 +631,8 @@ func (b *sourceFilesystemsBuilder) createModFs(
 		if md.isMainProject {
 			return b.p.AbsResourcesDir
 		}
-		return absPathify(files.FolderResources)
+		_, filename := absPathify(files.FolderResources)
+		return filename
 	}
 
 	if collector.overlayMounts == nil {
