@@ -3,13 +3,15 @@ package hugolib
 import (
 	"testing"
 
-	"github.com/stretchr/testify/require"
+	"github.com/gohugoio/hugo/resources/page"
+
+	qt "github.com/frankban/quicktest"
 )
 
 func TestMultihosts(t *testing.T) {
 	t.Parallel()
 
-	assert := require.New(t)
+	c := qt.New(t)
 
 	var configTemplate = `
 paginate = 1
@@ -55,12 +57,10 @@ languageName = "Nynorsk"
 
 	s1 := b.H.Sites[0]
 
-	assert.Equal([]string{"s1", "s2", "ens1", "ens2"}, s1.StaticDirs())
-
-	s1h := s1.getPage(KindHome)
-	assert.True(s1h.IsTranslated())
-	assert.Len(s1h.Translations(), 2)
-	assert.Equal("https://example.com/docs/", s1h.Permalink())
+	s1h := s1.getPage(page.KindHome)
+	c.Assert(s1h.IsTranslated(), qt.Equals, true)
+	c.Assert(len(s1h.Translations()), qt.Equals, 2)
+	c.Assert(s1h.Permalink(), qt.Equals, "https://example.com/docs/")
 
 	// For “regular multilingual” we kept the aliases pages with url in front matter
 	// as a literal value that we use as is.
@@ -68,10 +68,9 @@ languageName = "Nynorsk"
 	// For multihost, we never want any content in the root.
 	//
 	// check url in front matter:
-	pageWithURLInFrontMatter := s1.getPage(KindPage, "sect/doc3.en.md")
-	assert.NotNil(pageWithURLInFrontMatter)
-	assert.Equal("/superbob", pageWithURLInFrontMatter.URL())
-	assert.Equal("/docs/superbob/", pageWithURLInFrontMatter.RelPermalink())
+	pageWithURLInFrontMatter := s1.getPage(page.KindPage, "sect/doc3.en.md")
+	c.Assert(pageWithURLInFrontMatter, qt.Not(qt.IsNil))
+	c.Assert(pageWithURLInFrontMatter.RelPermalink(), qt.Equals, "/docs/superbob/")
 	b.AssertFileContent("public/en/superbob/index.html", "doc3|Hello|en")
 
 	// check alias:
@@ -79,13 +78,14 @@ languageName = "Nynorsk"
 	b.AssertFileContent("public/en/al/alias2/index.html", `content="0; url=https://example.com/docs/superbob/"`)
 
 	s2 := b.H.Sites[1]
-	assert.Equal([]string{"s1", "s2", "frs1", "frs2"}, s2.StaticDirs())
 
-	s2h := s2.getPage(KindHome)
-	assert.Equal("https://example.fr/", s2h.Permalink())
+	s2h := s2.getPage(page.KindHome)
+	c.Assert(s2h.Permalink(), qt.Equals, "https://example.fr/")
 
-	b.AssertFileContent("public/fr/index.html", "French Home Page")
-	b.AssertFileContent("public/en/index.html", "Default Home Page")
+	b.AssertFileContent("public/fr/index.html", "French Home Page", "String Resource: /docs/text/pipes.txt")
+	b.AssertFileContent("public/fr/text/pipes.txt", "Hugo Pipes")
+	b.AssertFileContent("public/en/index.html", "Default Home Page", "String Resource: /docs/text/pipes.txt")
+	b.AssertFileContent("public/en/text/pipes.txt", "Hugo Pipes")
 
 	// Check paginators
 	b.AssertFileContent("public/en/page/1/index.html", `refresh" content="0; url=https://example.com/docs/"`)
@@ -95,22 +95,19 @@ languageName = "Nynorsk"
 
 	// Check bundles
 
-	bundleEn := s1.getPage(KindPage, "bundles/b1/index.en.md")
-	require.NotNil(t, bundleEn)
-	require.Equal(t, "/docs/bundles/b1/", bundleEn.RelPermalink())
-	require.Equal(t, 1, len(bundleEn.Resources))
-	logoEn := bundleEn.Resources.GetByPrefix("logo")
-	require.NotNil(t, logoEn)
-	require.Equal(t, "/docs/bundles/b1/logo.png", logoEn.RelPermalink())
-	b.AssertFileContent("public/en/bundles/b1/logo.png", "PNG Data")
+	bundleEn := s1.getPage(page.KindPage, "bundles/b1/index.en.md")
+	c.Assert(bundleEn, qt.Not(qt.IsNil))
+	c.Assert(bundleEn.RelPermalink(), qt.Equals, "/docs/bundles/b1/")
+	c.Assert(len(bundleEn.Resources()), qt.Equals, 1)
 
-	bundleFr := s2.getPage(KindPage, "bundles/b1/index.md")
-	require.NotNil(t, bundleFr)
-	require.Equal(t, "/bundles/b1/", bundleFr.RelPermalink())
-	require.Equal(t, 1, len(bundleFr.Resources))
-	logoFr := bundleFr.Resources.GetByPrefix("logo")
-	require.NotNil(t, logoFr)
-	require.Equal(t, "/bundles/b1/logo.png", logoFr.RelPermalink())
+	b.AssertFileContent("public/en/bundles/b1/logo.png", "PNG Data")
+	b.AssertFileContent("public/en/bundles/b1/index.html", " image/png: /docs/bundles/b1/logo.png")
+
+	bundleFr := s2.getPage(page.KindPage, "bundles/b1/index.md")
+	c.Assert(bundleFr, qt.Not(qt.IsNil))
+	c.Assert(bundleFr.RelPermalink(), qt.Equals, "/bundles/b1/")
+	c.Assert(len(bundleFr.Resources()), qt.Equals, 1)
 	b.AssertFileContent("public/fr/bundles/b1/logo.png", "PNG Data")
+	b.AssertFileContent("public/fr/bundles/b1/index.html", " image/png: /bundles/b1/logo.png")
 
 }
