@@ -18,6 +18,8 @@ import (
 	"fmt"
 	"strconv"
 
+	"github.com/gohugoio/hugo/helpers"
+
 	"html/template"
 	"path"
 
@@ -43,7 +45,6 @@ import (
 	"github.com/gohugoio/hugo/output"
 
 	bp "github.com/gohugoio/hugo/bufferpool"
-	"github.com/gohugoio/hugo/helpers"
 	"github.com/gohugoio/hugo/tpl"
 )
 
@@ -195,7 +196,16 @@ type shortcode struct {
 }
 
 func (s shortcode) insertPlaceholder() bool {
-	return !s.doMarkup || s.info.Config.Version == 1
+	return !s.doMarkup || s.configVersion() == 1
+}
+
+func (s shortcode) configVersion() int {
+	if s.info == nil {
+		// Not set for inline shortcodes.
+		return 2
+	}
+
+	return s.info.ParseInfo().Config.Version
 }
 
 func (s shortcode) innerString() string {
@@ -293,7 +303,7 @@ func renderShortcode(
 			templStr := sc.innerString()
 
 			var err error
-			tmpl, err = s.TextTmpl.Parse(templName, templStr)
+			tmpl, err = s.TextTmpl().Parse(templName, templStr)
 			if err != nil {
 				fe := herrors.ToFileError("html", err)
 				l1, l2 := p.posOffset(sc.pos).LineNumber, fe.Position().LineNumber
@@ -304,14 +314,14 @@ func renderShortcode(
 		} else {
 			// Re-use of shortcode defined earlier in the same page.
 			var found bool
-			tmpl, found = s.TextTmpl.Lookup(templName)
+			tmpl, found = s.TextTmpl().Lookup(templName)
 			if !found {
 				return "", false, _errors.Errorf("no earlier definition of shortcode %q found", sc.name)
 			}
 		}
 	} else {
 		var found, more bool
-		tmpl, found, more = s.Tmpl.LookupVariant(sc.name, tplVariants)
+		tmpl, found, more = s.Tmpl().LookupVariant(sc.name, tplVariants)
 		if !found {
 			s.Log.ERROR.Printf("Unable to locate template for shortcode %q in page %q", sc.name, p.File().Path())
 			return "", false, nil
@@ -346,14 +356,15 @@ func renderShortcode(
 
 		// Pre Hugo 0.55 this was the behaviour even for the outer-most
 		// shortcode.
-		if sc.doMarkup && (level > 0 || sc.info.Config.Version == 1) {
-			newInner := s.ContentSpec.RenderBytes(&helpers.RenderingContext{
-				Content:      []byte(inner),
-				PageFmt:      p.m.markup,
-				Cfg:          p.Language(),
-				DocumentID:   p.File().UniqueID(),
-				DocumentName: p.File().Path(),
-				Config:       p.getRenderingConfig()})
+		if sc.doMarkup && (level > 0 || sc.configVersion() == 1) {
+			var err error
+			b, err := p.pageOutput.cp.renderContent([]byte(inner), false)
+
+			if err != nil {
+				return "", false, err
+			}
+
+			newInner := b.Bytes()
 
 			// If the type is “” (unknown) or “markdown”, we assume the markdown
 			// generation has been performed. Given the input: `a line`, markdown
@@ -384,7 +395,7 @@ func renderShortcode(
 
 	}
 
-	result, err := renderShortcodeWithPage(tmpl, data)
+	result, err := renderShortcodeWithPage(s.Tmpl(), tmpl, data)
 
 	if err != nil && sc.isInline {
 		fe := herrors.ToFileError("html", err)
@@ -397,7 +408,7 @@ func renderShortcode(
 }
 
 func (s *shortcodeHandler) hasShortcodes() bool {
-	return len(s.shortcodes) > 0
+	return s != nil && len(s.shortcodes) > 0
 }
 
 func (s *shortcodeHandler) renderShortcodesForPage(p *pageState, f output.Format) (map[string]string, bool, error) {
@@ -485,13 +496,13 @@ Loop:
 		case currItem.IsRightShortcodeDelim():
 			// we trust the template on this:
 			// if there's no inner, we're done
-			if !sc.isInline && !sc.info.IsInner {
+			if !sc.isInline && !sc.info.ParseInfo().IsInner {
 				return sc, nil
 			}
 
 		case currItem.IsShortcodeClose():
 			next := pt.Peek()
-			if !sc.isInline && !sc.info.IsInner {
+			if !sc.isInline && !sc.info.ParseInfo().IsInner {
 				if next.IsError() {
 					// return that error, more specific
 					continue
@@ -510,6 +521,15 @@ Loop:
 			return sc, nil
 		case currItem.IsText():
 			sc.inner = append(sc.inner, currItem.ValStr())
+		case currItem.Type == pageparser.TypeEmoji:
+			// TODO(bep) avoid the duplication of these "text cases", to prevent
+			// more of #6504 in the future.
+			val := currItem.ValStr()
+			if emoji := helpers.Emoji(val); emoji != nil {
+				sc.inner = append(sc.inner, string(emoji))
+			} else {
+				sc.inner = append(sc.inner, val)
+			}
 		case currItem.IsShortcodeName():
 
 			sc.name = currItem.ValStr()
@@ -517,12 +537,12 @@ Loop:
 			// Check if the template expects inner content.
 			// We pick the first template for an arbitrary output format
 			// if more than one. It is "all inner or no inner".
-			tmpl, found, _ := s.s.Tmpl.LookupVariant(sc.name, tpl.TemplateVariants{})
+			tmpl, found, _ := s.s.Tmpl().LookupVariant(sc.name, tpl.TemplateVariants{})
 			if !found {
 				return nil, _errors.Errorf("template for shortcode %q not found", sc.name)
 			}
 
-			sc.info = tmpl.(tpl.TemplateInfoProvider).TemplateInfo()
+			sc.info = tmpl.(tpl.Info)
 		case currItem.IsInlineShortcodeName():
 			sc.name = currItem.ValStr()
 			sc.isInline = true
@@ -616,11 +636,11 @@ func replaceShortcodeTokens(source []byte, replacements map[string]string) ([]by
 	return source, nil
 }
 
-func renderShortcodeWithPage(tmpl tpl.Template, data *ShortcodeWithPage) (string, error) {
+func renderShortcodeWithPage(h tpl.TemplateHandler, tmpl tpl.Template, data *ShortcodeWithPage) (string, error) {
 	buffer := bp.GetBuffer()
 	defer bp.PutBuffer(buffer)
 
-	err := tmpl.Execute(buffer, data)
+	err := h.Execute(tmpl, buffer, data)
 	if err != nil {
 		return "", _errors.Wrap(err, "failed to process shortcode")
 	}
