@@ -15,13 +15,15 @@ package hugolib
 
 import (
 	"fmt"
-	"io/ioutil"
+	"math/rand"
 	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
 	"testing"
 
+	"github.com/gohugoio/hugo/resources/page"
+
 	qt "github.com/frankban/quicktest"
 )
 
@@ -31,9 +33,77 @@ type siteBenchmarkTestcase struct {
 	check  func(s *sitesBuilder)
 }
 
-func getBenchmarkSiteNewTestCases() []siteBenchmarkTestcase {
+func getBenchmarkSiteDeepContent(b testing.TB) *sitesBuilder {
+	pageContent := func(size int) string {
+		return getBenchmarkTestDataPageContentForMarkdown(size, "", benchmarkMarkdownSnippets)
+	}
 
-	const markdownSnippets = `
+	sb := newTestSitesBuilder(b).WithConfigFile("toml", `
+baseURL = "https://example.com"
+
+[languages]
+[languages.en]
+weight=1
+contentDir="content/en"
+[languages.fr]
+weight=2
+contentDir="content/fr"
+[languages.no]
+weight=3
+contentDir="content/no"
+[languages.sv]
+weight=4
+contentDir="content/sv"
+			
+`)
+
+	createContent := func(dir, name string) {
+		sb.WithContent(filepath.Join("content", dir, name), pageContent(1))
+	}
+
+	createBundledFiles := func(dir string) {
+		sb.WithContent(filepath.Join("content", dir, "data.json"), `{ "hello": "world" }`)
+		for i := 1; i <= 3; i++ {
+			sb.WithContent(filepath.Join("content", dir, fmt.Sprintf("page%d.md", i)), pageContent(1))
+		}
+	}
+
+	for _, lang := range []string{"en", "fr", "no", "sv"} {
+		for level := 1; level <= 5; level++ {
+			sectionDir := path.Join(lang, strings.Repeat("section/", level))
+			createContent(sectionDir, "_index.md")
+			createBundledFiles(sectionDir)
+			for i := 1; i <= 3; i++ {
+				leafBundleDir := path.Join(sectionDir, fmt.Sprintf("bundle%d", i))
+				createContent(leafBundleDir, "index.md")
+				createBundledFiles(path.Join(leafBundleDir, "assets1"))
+				createBundledFiles(path.Join(leafBundleDir, "assets1", "assets2"))
+			}
+		}
+	}
+
+	return sb
+}
+
+func getBenchmarkTestDataPageContentForMarkdown(size int, category, markdown string) string {
+	base := `---
+title: "My Page"
+%s
+---
+
+My page content.
+`
+
+	var categoryKey string
+	if category != "" {
+		categoryKey = fmt.Sprintf("categories: [%s]", category)
+	}
+	base = fmt.Sprintf(base, categoryKey)
+
+	return base + strings.Repeat(markdown, size)
+}
+
+const benchmarkMarkdownSnippets = `
 
 ## Links
 
@@ -47,18 +117,14 @@ This is [Relative](/all-is-relative).
 See my [About](/about/) page for details. 
 `
 
-	pageContentForMarkdown := func(size int, markdown string) string {
-		return `---
-title: "My Page"
----
+func getBenchmarkSiteNewTestCases() []siteBenchmarkTestcase {
 
-My page content.
-
-` + strings.Repeat(markdown, size)
+	pageContentWithCategory := func(size int, category string) string {
+		return getBenchmarkTestDataPageContentForMarkdown(size, category, benchmarkMarkdownSnippets)
 	}
 
 	pageContent := func(size int) string {
-		return pageContentForMarkdown(size, markdownSnippets)
+		return getBenchmarkTestDataPageContentForMarkdown(size, "", benchmarkMarkdownSnippets)
 	}
 
 	config := `
@@ -121,63 +187,6 @@ Some content.
 				s.AssertFileContent("public/tags/ta3/index.html", "|ta3|")
 			},
 		},
-		{"Markdown", func(b testing.TB) *sitesBuilder {
-			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
-title = "What is Markdown"
-baseURL = "https://example.com"
-
-`)
-
-			data, err := ioutil.ReadFile(filepath.FromSlash("testdata/what-is-markdown.md"))
-			sb.Assert(err, qt.IsNil)
-			datastr := string(data)
-			getContent := func(i int) string {
-				return fmt.Sprintf(`---
-title: "Page %d"
----
-
-`, i) + datastr
-
-			}
-			for i := 1; i <= 100; i++ {
-				sb.WithContent(fmt.Sprintf("content/page%d.md", i), getContent(i))
-			}
-
-			return sb
-		},
-			func(s *sitesBuilder) {
-				s.Assert(s.CheckExists("public/page8/index.html"), qt.Equals, true)
-			},
-		},
-		{"Markdown with custom link handler", func(b testing.TB) *sitesBuilder {
-			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
-title = "What is Markdown"
-baseURL = "https://example.com"
-
-`)
-
-			sb.WithTemplatesAdded("_default/_markup/render-link.html", `<a href="{{ .Destination | safeURL }}#custom">CUSTOM LINK</a>`)
-			data, err := ioutil.ReadFile(filepath.FromSlash("testdata/what-is-markdown.md"))
-			sb.Assert(err, qt.IsNil)
-			datastr := string(data)
-			getContent := func(i int) string {
-				return fmt.Sprintf(`---
-title: "Page %d"
----
-
-`, i) + datastr
-
-			}
-			for i := 1; i <= 100; i++ {
-				sb.WithContent(fmt.Sprintf("content/page%d.md", i), getContent(i))
-			}
-
-			return sb
-		},
-			func(s *sitesBuilder) {
-				s.Assert(s.CheckExists("public/page8/index.html"), qt.Equals, true)
-			},
-		},
 		{"Canonify URLs", func(b testing.TB) *sitesBuilder {
 			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
 title = "Canon"
@@ -195,26 +204,44 @@ canonifyURLs = true
 				s.AssertFileContent("public/page8/index.html", "https://example.com/about/")
 			},
 		},
-		{"Code Fences", func(b testing.TB) *sitesBuilder {
-			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
-title = "Code"
-baseURL = "https://example.com"
-pygmentsCodeFences = true
 
-`)
-			markdown := "\n```bash\n" + `echo "Hugo Rocks!"` + "\n```\n\n"
-
-			for i := 1; i <= 100; i++ {
-				sb.WithContent(fmt.Sprintf("content/page%d.md", i), pageContentForMarkdown(i, markdown))
-			}
-
-			return sb
+		{"Deep content tree", func(b testing.TB) *sitesBuilder {
+			return getBenchmarkSiteDeepContent(b)
 		},
 			func(s *sitesBuilder) {
-				s.AssertFileContent("public/page8/index.html", `<div class="highlight"><pre style="color:#f8f8f2;background-color:#272822;-moz-tab-size:4;-o-tab-size:4;tab-size:4"><code class="language-bash" data-lang="bash">echo <span style="color:#e6db74">&#34;Hugo Rocks!&#34;</span>`)
+				s.CheckExists("public/blog/mybundle/index.html")
+				s.Assert(len(s.H.Sites), qt.Equals, 4)
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, len(s.H.Sites[1].RegularPages()))
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, 30)
+
 			},
 		},
-		{"Deep content tree", func(b testing.TB) *sitesBuilder {
+		{"Many HTML templates", func(b testing.TB) *sitesBuilder {
+
+			pageTemplateTemplate := `
+<!DOCTYPE html>
+<html>
+  <head>
+    <meta charset="utf-8">
+    <title>{{ if not .IsPage }}{{ .Title }}{{ else }}{{ printf "Site: %s" site.Title }}{{ end }}</title>
+    <style>
+     body {
+       margin: 3rem;
+     }
+    </style>
+  </head>
+  <body>
+    <div class="page">{{ .Content }}</div>
+    <ul>
+    {{ with .Pages }}
+    {{ range . }}
+    <li><a href="{{ .RelPermalink }}">{{ .LinkTitle }} {{ if not .IsNode }} (Page){{ end }}</a></li>
+    {{ end }}
+    {{ end }}
+    </ul>
+  </body>
+</html>
+`
 
 			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
 baseURL = "https://example.com"
@@ -239,6 +266,85 @@ contentDir="content/sv"
 				sb.WithContent(filepath.Join("content", dir, name), pageContent(1))
 			}
 
+			for _, lang := range []string{"en", "fr", "no", "sv"} {
+				sb.WithTemplatesAdded(fmt.Sprintf("_default/single.%s.html", lang), pageTemplateTemplate)
+				sb.WithTemplatesAdded(fmt.Sprintf("_default/list.%s.html", lang), pageTemplateTemplate)
+
+				for level := 1; level <= 5; level++ {
+					sectionDir := path.Join(lang, strings.Repeat("section/", level))
+					createContent(sectionDir, "_index.md")
+					for i := 1; i <= 3; i++ {
+						leafBundleDir := path.Join(sectionDir, fmt.Sprintf("bundle%d", i))
+						createContent(leafBundleDir, "index.md")
+					}
+				}
+			}
+
+			return sb
+		},
+			func(s *sitesBuilder) {
+				s.CheckExists("public/blog/mybundle/index.html")
+				s.Assert(len(s.H.Sites), qt.Equals, 4)
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, len(s.H.Sites[1].RegularPages()))
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, 15)
+
+			},
+		},
+		{"Page collections", func(b testing.TB) *sitesBuilder {
+
+			pageTemplateTemplate := `
+{{ if .IsNode }}
+{{ len .Paginator.Pages }}
+{{ end }}
+{{ len .Sections }}
+{{ len .Pages }}
+{{ len .RegularPages }}
+{{ len .Resources }}
+{{ len site.RegularPages }}
+{{ len site.Pages }}
+{{ with .NextInSection }}Next in section: {{ .RelPermalink }}{{ end }}
+{{ with .PrevInSection }}Prev in section: {{ .RelPermalink }}{{ end }}
+{{ with .Next }}Next: {{ .RelPermalink }}{{ end }}
+{{ with .Prev }}Prev: {{ .RelPermalink }}{{ end }}
+`
+
+			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
+baseURL = "https://example.com"
+
+[languages]
+[languages.en]
+weight=1
+contentDir="content/en"
+[languages.fr]
+weight=2
+contentDir="content/fr"
+[languages.no]
+weight=3
+contentDir="content/no"
+[languages.sv]
+weight=4
+contentDir="content/sv"
+			
+`)
+
+			sb.WithTemplates("index.html", pageTemplateTemplate)
+			sb.WithTemplates("_default/single.html", pageTemplateTemplate)
+			sb.WithTemplates("_default/list.html", pageTemplateTemplate)
+
+			r := rand.New(rand.NewSource(99))
+
+			createContent := func(dir, name string) {
+				var content string
+				if strings.Contains(name, "_index") {
+					content = pageContent(1)
+
+				} else {
+					content = pageContentWithCategory(1, fmt.Sprintf("category%d", r.Intn(5)+1))
+				}
+
+				sb.WithContent(filepath.Join("content", dir, name), content)
+			}
+
 			createBundledFiles := func(dir string) {
 				sb.WithContent(filepath.Join("content", dir, "data.json"), `{ "hello": "world" }`)
 				for i := 1; i <= 3; i++ {
@@ -247,11 +353,11 @@ contentDir="content/sv"
 			}
 
 			for _, lang := range []string{"en", "fr", "no", "sv"} {
-				for level := 1; level <= 5; level++ {
+				for level := 1; level <= r.Intn(5)+1; level++ {
 					sectionDir := path.Join(lang, strings.Repeat("section/", level))
 					createContent(sectionDir, "_index.md")
 					createBundledFiles(sectionDir)
-					for i := 1; i <= 3; i++ {
+					for i := 1; i <= r.Intn(20)+1; i++ {
 						leafBundleDir := path.Join(sectionDir, fmt.Sprintf("bundle%d", i))
 						createContent(leafBundleDir, "index.md")
 						createBundledFiles(path.Join(leafBundleDir, "assets1"))
@@ -265,8 +371,53 @@ contentDir="content/sv"
 			func(s *sitesBuilder) {
 				s.CheckExists("public/blog/mybundle/index.html")
 				s.Assert(len(s.H.Sites), qt.Equals, 4)
-				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, len(s.H.Sites[1].RegularPages()))
-				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, 30)
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, 26)
+
+			},
+		},
+		{"List terms", func(b testing.TB) *sitesBuilder {
+
+			pageTemplateTemplate := `
+<ul>
+    {{ range (.GetTerms "categories") }}
+        <li><a href="{{ .Permalink }}">{{ .LinkTitle }}</a></li>
+   {{ end }}
+</ul>
+`
+
+			sb := newTestSitesBuilder(b).WithConfigFile("toml", `
+baseURL = "https://example.com"
+`)
+
+			sb.WithTemplates("_default/single.html", pageTemplateTemplate)
+
+			r := rand.New(rand.NewSource(99))
+
+			createContent := func(dir, name string) {
+				var content string
+				if strings.Contains(name, "_index") {
+					content = pageContent(1)
+				} else {
+					content = pageContentWithCategory(1, fmt.Sprintf("category%d", r.Intn(5)+1))
+					sb.WithContent(filepath.Join("content", dir, name), content)
+				}
+			}
+
+			for level := 1; level <= r.Intn(5)+1; level++ {
+				sectionDir := path.Join(strings.Repeat("section/", level))
+				createContent(sectionDir, "_index.md")
+				for i := 1; i <= r.Intn(33); i++ {
+					leafBundleDir := path.Join(sectionDir, fmt.Sprintf("bundle%d", i))
+					createContent(leafBundleDir, "index.md")
+				}
+			}
+
+			return sb
+		},
+			func(s *sitesBuilder) {
+				s.AssertFileContent("public/section/bundle8/index.html", ` <li><a href="https://example.com/categories/category1/">category1</a></li>`)
+				s.Assert(len(s.H.Sites), qt.Equals, 1)
+				s.Assert(len(s.H.Sites[0].RegularPages()), qt.Equals, 35)
 
 			},
 		},
@@ -294,26 +445,93 @@ func TestBenchmarkSiteNew(b *testing.T) {
 	}
 }
 
-// TODO(bep) eventually remove the old (too complicated setup).
+func TestBenchmarkSiteDeepContentEdit(t *testing.T) {
+	b := getBenchmarkSiteDeepContent(t).Running()
+	b.Build(BuildCfg{})
+
+	p := b.H.Sites[0].RegularPages()[12]
+
+	b.EditFiles(p.File().Filename(), fmt.Sprintf(`---
+title: %s
+---
+
+Edited!!`, p.Title()))
+
+	counters := &testCounters{}
+
+	b.Build(BuildCfg{testCounters: counters})
+
+	// We currently rebuild all the language versions of the same content file.
+	// We could probably optimize that case, but it's not trivial.
+	b.Assert(int(counters.contentRenderCounter), qt.Equals, 4)
+	b.AssertFileContent("public"+p.RelPermalink()+"index.html", "Edited!!")
+
+}
+
 func BenchmarkSiteNew(b *testing.B) {
+	rnd := rand.New(rand.NewSource(32))
 	benchmarks := getBenchmarkSiteNewTestCases()
-
-	for _, bm := range benchmarks {
-		b.Run(bm.name, func(b *testing.B) {
-			sites := make([]*sitesBuilder, b.N)
-			for i := 0; i < b.N; i++ {
-				sites[i] = bm.create(b)
+	for _, edit := range []bool{true, false} {
+		for _, bm := range benchmarks {
+			name := bm.name
+			if edit {
+				name = "Edit_" + name
+			} else {
+				name = "Regular_" + name
 			}
-
-			b.ResetTimer()
-			for i := 0; i < b.N; i++ {
-				s := sites[i]
-				err := s.BuildE(BuildCfg{})
-				if err != nil {
-					b.Fatal(err)
+			b.Run(name, func(b *testing.B) {
+				sites := make([]*sitesBuilder, b.N)
+				for i := 0; i < b.N; i++ {
+					sites[i] = bm.create(b)
+					if edit {
+						sites[i].Running()
+					}
 				}
-				bm.check(s)
-			}
-		})
+
+				b.ResetTimer()
+				for i := 0; i < b.N; i++ {
+					if edit {
+						b.StopTimer()
+					}
+					s := sites[i]
+					err := s.BuildE(BuildCfg{})
+					if err != nil {
+						b.Fatal(err)
+					}
+					bm.check(s)
+
+					if edit {
+						if edit {
+							b.StartTimer()
+						}
+						// Edit a random page in a random language.
+						pages := s.H.Sites[rnd.Intn(len(s.H.Sites))].Pages()
+						var p page.Page
+						count := 0
+						for {
+							count++
+							if count > 100 {
+								panic("infinite loop")
+							}
+							p = pages[rnd.Intn(len(pages))]
+							if !p.File().IsZero() {
+								break
+							}
+						}
+
+						s.EditFiles(p.File().Filename(), fmt.Sprintf(`---
+title: %s
+---
+
+Edited!!`, p.Title()))
+
+						err := s.BuildE(BuildCfg{})
+						if err != nil {
+							b.Fatal(err)
+						}
+					}
+				}
+			})
+		}
 	}
 }
