@@ -3,12 +3,16 @@ package hugolib
 import (
 	"image/jpeg"
 	"io"
+	"math/rand"
 	"path/filepath"
 	"runtime"
 	"strconv"
 	"testing"
+	"time"
 	"unicode/utf8"
 
+	"github.com/gohugoio/hugo/htesting"
+
 	"github.com/gohugoio/hugo/output"
 
 	"github.com/gohugoio/hugo/parser/metadecoders"
@@ -63,7 +67,7 @@ type sitesBuilder struct {
 	*qt.C
 
 	logger *loggers.Logger
-
+	rnd    *rand.Rand
 	dumper litter.Options
 
 	// Used to test partial rebuilds.
@@ -89,17 +93,22 @@ type sitesBuilder struct {
 
 	addNothing bool
 	// Base data/content
-	contentFilePairs  []string
-	templateFilePairs []string
-	i18nFilePairs     []string
-	dataFilePairs     []string
+	contentFilePairs  []filenameContent
+	templateFilePairs []filenameContent
+	i18nFilePairs     []filenameContent
+	dataFilePairs     []filenameContent
 
 	// Additional data/content.
 	// As in "use the base, but add these on top".
-	contentFilePairsAdded  []string
-	templateFilePairsAdded []string
-	i18nFilePairsAdded     []string
-	dataFilePairsAdded     []string
+	contentFilePairsAdded  []filenameContent
+	templateFilePairsAdded []filenameContent
+	i18nFilePairsAdded     []filenameContent
+	dataFilePairsAdded     []filenameContent
+}
+
+type filenameContent struct {
+	filename string
+	content  string
 }
 
 func newTestSitesBuilder(t testing.TB) *sitesBuilder {
@@ -112,7 +121,8 @@ func newTestSitesBuilder(t testing.TB) *sitesBuilder {
 		Separator:         " ",
 	}
 
-	return &sitesBuilder{T: t, C: qt.New(t), Fs: fs, configFormat: "toml", dumper: litterOptions}
+	return &sitesBuilder{T: t, C: qt.New(t), Fs: fs, configFormat: "toml",
+		dumper: litterOptions, rnd: rand.New(rand.NewSource(time.Now().Unix()))}
 }
 
 func newTestSitesBuilderFromDepsCfg(t testing.TB, d deps.DepsCfg) *sitesBuilder {
@@ -124,7 +134,7 @@ func newTestSitesBuilderFromDepsCfg(t testing.TB, d deps.DepsCfg) *sitesBuilder
 		Separator:         " ",
 	}
 
-	b := &sitesBuilder{T: t, C: c, depsCfg: d, Fs: d.Fs, dumper: litterOptions}
+	b := &sitesBuilder{T: t, C: c, depsCfg: d, Fs: d.Fs, dumper: litterOptions, rnd: rand.New(rand.NewSource(time.Now().Unix()))}
 	workingDir := d.Cfg.GetString("workingDir")
 
 	b.WithWorkingDir(workingDir)
@@ -224,6 +234,9 @@ func (s *sitesBuilder) WithSourceFile(filenameContent ...string) *sitesBuilder {
 
 func (s *sitesBuilder) absFilename(filename string) string {
 	filename = filepath.FromSlash(filename)
+	if filepath.IsAbs(filename) {
+		return filename
+	}
 	if s.workingDir != "" && !strings.HasPrefix(filename, s.workingDir) {
 		filename = filepath.Join(s.workingDir, filename)
 	}
@@ -346,43 +359,62 @@ func (s *sitesBuilder) WithSunset(in string) {
 	src.Close()
 }
 
+func (s *sitesBuilder) createFilenameContent(pairs []string) []filenameContent {
+	var slice []filenameContent
+	s.appendFilenameContent(&slice, pairs...)
+	return slice
+}
+
+func (s *sitesBuilder) appendFilenameContent(slice *[]filenameContent, pairs ...string) {
+	if len(pairs)%2 != 0 {
+		panic("file content mismatch")
+	}
+	for i := 0; i < len(pairs); i += 2 {
+		c := filenameContent{
+			filename: pairs[i],
+			content:  pairs[i+1],
+		}
+		*slice = append(*slice, c)
+	}
+}
+
 func (s *sitesBuilder) WithContent(filenameContent ...string) *sitesBuilder {
-	s.contentFilePairs = append(s.contentFilePairs, filenameContent...)
+	s.appendFilenameContent(&s.contentFilePairs, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithContentAdded(filenameContent ...string) *sitesBuilder {
-	s.contentFilePairsAdded = append(s.contentFilePairsAdded, filenameContent...)
+	s.appendFilenameContent(&s.contentFilePairsAdded, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithTemplates(filenameContent ...string) *sitesBuilder {
-	s.templateFilePairs = append(s.templateFilePairs, filenameContent...)
+	s.appendFilenameContent(&s.templateFilePairs, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithTemplatesAdded(filenameContent ...string) *sitesBuilder {
-	s.templateFilePairsAdded = append(s.templateFilePairsAdded, filenameContent...)
+	s.appendFilenameContent(&s.templateFilePairsAdded, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithData(filenameContent ...string) *sitesBuilder {
-	s.dataFilePairs = append(s.dataFilePairs, filenameContent...)
+	s.appendFilenameContent(&s.dataFilePairs, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithDataAdded(filenameContent ...string) *sitesBuilder {
-	s.dataFilePairsAdded = append(s.dataFilePairsAdded, filenameContent...)
+	s.appendFilenameContent(&s.dataFilePairsAdded, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithI18n(filenameContent ...string) *sitesBuilder {
-	s.i18nFilePairs = append(s.i18nFilePairs, filenameContent...)
+	s.appendFilenameContent(&s.i18nFilePairs, filenameContent...)
 	return s
 }
 
 func (s *sitesBuilder) WithI18nAdded(filenameContent ...string) *sitesBuilder {
-	s.i18nFilePairsAdded = append(s.i18nFilePairsAdded, filenameContent...)
+	s.appendFilenameContent(&s.i18nFilePairsAdded, filenameContent...)
 	return s
 }
 
@@ -406,15 +438,18 @@ func (s *sitesBuilder) RemoveFiles(filenames ...string) *sitesBuilder {
 	return s
 }
 
-func (s *sitesBuilder) writeFilePairs(folder string, filenameContent []string) *sitesBuilder {
-	if len(filenameContent)%2 != 0 {
-		s.Fatalf("expect filenameContent for %q in pairs (%d)", folder, len(filenameContent))
-	}
-	for i := 0; i < len(filenameContent); i += 2 {
-		filename, content := filenameContent[i], filenameContent[i+1]
+func (s *sitesBuilder) writeFilePairs(folder string, files []filenameContent) *sitesBuilder {
+	// We have had some "filesystem ordering" bugs that we have not discovered in
+	// our tests running with the in memory filesystem.
+	// That file system is backed by a map so not sure how this helps, but some
+	// randomness in tests doesn't hurt.
+	// TODO(bep) this turns out to be more confusing than helpful.
+	//s.rnd.Shuffle(len(files), func(i, j int) { files[i], files[j] = files[j], files[i] })
+
+	for _, fc := range files {
 		target := folder
 		// TODO(bep) clean  up this magic.
-		if strings.HasPrefix(filename, folder) {
+		if strings.HasPrefix(fc.filename, folder) {
 			target = ""
 		}
 
@@ -422,14 +457,14 @@ func (s *sitesBuilder) writeFilePairs(folder string, filenameContent []string) *
 			target = filepath.Join(s.workingDir, target)
 		}
 
-		writeSource(s.T, s.Fs, filepath.Join(target, filename), content)
+		writeSource(s.T, s.Fs, filepath.Join(target, fc.filename), fc.content)
 	}
 	return s
 }
 
 func (s *sitesBuilder) CreateSites() *sitesBuilder {
 	if err := s.CreateSitesE(); err != nil {
-		herrors.PrintStackTrace(err)
+		herrors.PrintStackTraceFromErr(err)
 		s.Fatalf("Failed to create sites: %s", err)
 	}
 
@@ -466,6 +501,7 @@ func (s *sitesBuilder) CreateSitesE() error {
 			for _, dir := range []string{
 				"content/sect",
 				"layouts/_default",
+				"layouts/_default/_markup",
 				"layouts/partials",
 				"layouts/shortcodes",
 				"data",
@@ -568,7 +604,7 @@ func (s *sitesBuilder) build(cfg BuildCfg, shouldFail bool) *sitesBuilder {
 		}
 	}
 	if err != nil && !shouldFail {
-		herrors.PrintStackTrace(err)
+		herrors.PrintStackTraceFromErr(err)
 		s.Fatalf("Build failed: %s", err)
 	} else if err == nil && shouldFail {
 		s.Fatalf("Expected error")
@@ -636,16 +672,17 @@ hello:
 	)
 
 	if len(s.contentFilePairs) == 0 {
-		s.writeFilePairs("content", defaultContent)
+		s.writeFilePairs("content", s.createFilenameContent(defaultContent))
 	}
+
 	if len(s.templateFilePairs) == 0 {
-		s.writeFilePairs("layouts", defaultTemplates)
+		s.writeFilePairs("layouts", s.createFilenameContent(defaultTemplates))
 	}
 	if len(s.dataFilePairs) == 0 {
-		s.writeFilePairs("data", defaultData)
+		s.writeFilePairs("data", s.createFilenameContent(defaultData))
 	}
 	if len(s.i18nFilePairs) == 0 {
-		s.writeFilePairs("i18n", defaultI18n)
+		s.writeFilePairs("i18n", s.createFilenameContent(defaultI18n))
 	}
 }
 
@@ -689,6 +726,7 @@ func (s *sitesBuilder) AssertImage(width, height int, filename string) {
 	s.Assert(err, qt.IsNil)
 	defer f.Close()
 	cfg, err := jpeg.DecodeConfig(f)
+	s.Assert(err, qt.IsNil)
 	s.Assert(cfg.Width, qt.Equals, width)
 	s.Assert(cfg.Height, qt.Equals, height)
 }
@@ -715,7 +753,7 @@ func (s *sitesBuilder) AssertObject(expected string, object interface{}) {
 
 	if expected != got {
 		fmt.Println(got)
-		diff := helpers.DiffStrings(expected, got)
+		diff := htesting.DiffStrings(expected, got)
 		s.Fatalf("diff:\n%s\nexpected\n%s\ngot\n%s", diff, expected, got)
 	}
 }
@@ -734,6 +772,18 @@ func (s *sitesBuilder) CheckExists(filename string) bool {
 	return destinationExists(s.Fs, filepath.Clean(filename))
 }
 
+func (s *sitesBuilder) GetPage(ref string) page.Page {
+	p, err := s.H.Sites[0].getPageNew(nil, ref)
+	s.Assert(err, qt.IsNil)
+	return p
+}
+
+func (s *sitesBuilder) GetPageRel(p page.Page, ref string) page.Page {
+	p, err := s.H.Sites[0].getPageNew(p, ref)
+	s.Assert(err, qt.IsNil)
+	return p
+}
+
 func newTestHelper(cfg config.Provider, fs *hugofs.Fs, t testing.TB) testHelper {
 	return testHelper{
 		Cfg: cfg,
@@ -971,7 +1021,7 @@ func printStringIndexes(s string) {
 }
 
 func isCI() bool {
-	return os.Getenv("CI") != ""
+	return (os.Getenv("CI") != "" || os.Getenv("CI_LOCAL") != "") && os.Getenv("CIRCLE_BRANCH") == ""
 }
 
 // See https://github.com/golang/go/issues/19280
@@ -990,3 +1040,18 @@ func skipSymlink(t *testing.T) {
 	}
 
 }
+
+func captureStderr(f func() error) (string, error) {
+	old := os.Stderr
+	r, w, _ := os.Pipe()
+	os.Stderr = w
+
+	err := f()
+
+	w.Close()
+	os.Stderr = old
+
+	var buf bytes.Buffer
+	io.Copy(&buf, r)
+	return buf.String(), err
+}
