@@ -19,7 +19,9 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
+	"github.com/bep/debounce"
 	"github.com/gohugoio/hugo/common/loggers"
 
 	"github.com/spf13/cast"
@@ -95,12 +97,13 @@ func (h *Client) collect(tidy bool) (ModulesConfig, *collector) {
 		return ModulesConfig{}, c
 	}
 
-	if !c.skipTidy && tidy {
+	// https://github.com/gohugoio/hugo/issues/6115
+	/*if !c.skipTidy && tidy {
 		if err := h.tidy(c.modules, true); err != nil {
 			c.err = err
 			return ModulesConfig{}, c
 		}
-	}
+	}*/
 
 	return ModulesConfig{
 		AllModules:        c.modules,
@@ -190,6 +193,11 @@ func (c *collector) initModules() error {
 	c.collected = &collected{
 		seen:     make(map[string]bool),
 		vendored: make(map[string]vendoredModule),
+		gomods:   goModules{},
+	}
+
+	if !c.ccfg.IgnoreVendor && c.isVendored(c.ccfg.WorkingDir) {
+		return nil
 	}
 
 	// We may fail later if we don't find the mods.
@@ -249,8 +257,7 @@ func (c *collector) add(owner *moduleAdapter, moduleImport Import, disabled bool
 		}
 
 		if moduleDir == "" {
-
-			if c.GoModulesFilename != "" && c.isProbablyModule(modulePath) {
+			if c.GoModulesFilename != "" && isProbablyModule(modulePath) {
 				// Try to "go get" it and reload the module configuration.
 				if err := c.Get(modulePath); err != nil {
 					return nil, err
@@ -300,10 +307,6 @@ func (c *collector) add(owner *moduleAdapter, moduleImport Import, disabled bool
 		ma.path = modulePath
 	}
 
-	if err := ma.validateAndApplyDefaults(c.fs); err != nil {
-		return nil, err
-	}
-
 	if !moduleImport.IgnoreConfig {
 		if err := c.applyThemeConfig(ma); err != nil {
 			return nil, err
@@ -412,9 +415,10 @@ func (c *collector) applyThemeConfig(tc *moduleAdapter) error {
 		}
 		themeCfg, err = metadecoders.Default.UnmarshalToMap(data, metadecoders.TOML)
 		if err != nil {
-			return errors.Wrapf(err, "failed to read module config for %q in %q", tc.Path(), themeTOML)
+			c.logger.WARN.Printf("Failed to read module config for %q in %q: %s", tc.Path(), themeTOML, err)
+		} else {
+			maps.ToLower(themeCfg)
 		}
-		maps.ToLower(themeCfg)
 	}
 
 	if hasConfig {
@@ -466,6 +470,13 @@ func (c *collector) applyThemeConfig(tc *moduleAdapter) error {
 }
 
 func (c *collector) collect() {
+	defer c.logger.PrintTimerIfDelayed(time.Now(), "hugo: collected modules")
+	d := debounce.New(2 * time.Second)
+	d(func() {
+		c.logger.FEEDBACK.Println("hugo: downloading modules …")
+	})
+	defer d(func() {})
+
 	if err := c.initModules(); err != nil {
 		c.err = err
 		return
@@ -483,6 +494,11 @@ func (c *collector) collect() {
 
 }
 
+func (c *collector) isVendored(dir string) bool {
+	_, err := c.fs.Stat(filepath.Join(dir, vendord, vendorModulesFilename))
+	return err == nil
+}
+
 func (c *collector) collectModulesTXT(owner Module) error {
 	vendorDir := filepath.Join(owner.Dir(), vendord)
 	filename := filepath.Join(vendorDir, vendorModulesFilename)
@@ -532,7 +548,7 @@ func (c *collector) loadModules() error {
 	return nil
 }
 
-func (c *collector) normalizeMounts(owner Module, mounts []Mount) ([]Mount, error) {
+func (c *collector) normalizeMounts(owner *moduleAdapter, mounts []Mount) ([]Mount, error) {
 	var out []Mount
 	dir := owner.Dir()
 
@@ -546,8 +562,16 @@ func (c *collector) normalizeMounts(owner Module, mounts []Mount) ([]Mount, erro
 		mnt.Source = filepath.Clean(mnt.Source)
 		mnt.Target = filepath.Clean(mnt.Target)
 
+		var sourceDir string
+
+		if owner.projectMod && filepath.IsAbs(mnt.Source) {
+			// Abs paths in the main project is allowed.
+			sourceDir = mnt.Source
+		} else {
+			sourceDir = filepath.Join(dir, mnt.Source)
+		}
+
 		// Verify that Source exists
-		sourceDir := filepath.Join(dir, mnt.Source)
 		_, err := c.fs.Stat(sourceDir)
 		if err != nil {
 			continue
