@@ -1,4 +1,4 @@
-// Copyright 2017-present The Hugo Authors. All rights reserved.
+// Copyright 2019 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -69,19 +69,31 @@ type Format struct {
 	// Note that we use the term "alternative" and not "alternate" here, as it
 	// does not necessarily replace the other format, it is an alternative representation.
 	NotAlternative bool `json:"notAlternative"`
+
+	// Setting this will make this output format control the value of
+	// .Permalink and .RelPermalink for a rendered Page.
+	// If not set, these values will point to the main (first) output format
+	// configured. That is probably the behaviour you want in most situations,
+	// as you probably don't want to link back to the RSS version of a page, as an
+	// example. AMP would, however, be a good example of an output format where this
+	// behaviour is wanted.
+	Permalinkable bool `json:"permalinkable"`
+
+	// Setting this to a non-zero value will be used as the first sort criteria.
+	Weight int `json:"weight"`
 }
 
+// An ordered list of built-in output formats.
 var (
-	// An ordered list of built-in output formats
-	//
-	// See https://www.ampproject.org/learn/overview/
 	AMPFormat = Format{
-		Name:      "AMP",
-		MediaType: media.HTMLType,
-		BaseName:  "index",
-		Path:      "amp",
-		Rel:       "amphtml",
-		IsHTML:    true,
+		Name:          "AMP",
+		MediaType:     media.HTMLType,
+		BaseName:      "index",
+		Path:          "amp",
+		Rel:           "amphtml",
+		IsHTML:        true,
+		Permalinkable: true,
+		// See https://www.ampproject.org/learn/overview/
 	}
 
 	CalendarFormat = Format{
@@ -110,11 +122,16 @@ var (
 	}
 
 	HTMLFormat = Format{
-		Name:      "HTML",
-		MediaType: media.HTMLType,
-		BaseName:  "index",
-		Rel:       "canonical",
-		IsHTML:    true,
+		Name:          "HTML",
+		MediaType:     media.HTMLType,
+		BaseName:      "index",
+		Rel:           "canonical",
+		IsHTML:        true,
+		Permalinkable: true,
+
+		// Weight will be used as first sort criteria. HTML will, by default,
+		// be rendered first, but set it to 10 so it's easy to put one above it.
+		Weight: 10,
 	}
 
 	JSONFormat = Format{
@@ -150,6 +167,7 @@ var (
 	}
 )
 
+// DefaultFormats contains the default output formats supported by Hugo.
 var DefaultFormats = Formats{
 	AMPFormat,
 	CalendarFormat,
@@ -166,11 +184,24 @@ func init() {
 	sort.Sort(DefaultFormats)
 }
 
+// Formats is a slice of Format.
 type Formats []Format
 
-func (formats Formats) Len() int           { return len(formats) }
-func (formats Formats) Swap(i, j int)      { formats[i], formats[j] = formats[j], formats[i] }
-func (formats Formats) Less(i, j int) bool { return formats[i].Name < formats[j].Name }
+func (formats Formats) Len() int      { return len(formats) }
+func (formats Formats) Swap(i, j int) { formats[i], formats[j] = formats[j], formats[i] }
+func (formats Formats) Less(i, j int) bool {
+	fi, fj := formats[i], formats[j]
+	if fi.Weight == fj.Weight {
+		return fi.Name < fj.Name
+	}
+
+	if fj.Weight == 0 {
+		return true
+	}
+
+	return fi.Weight > 0 && fi.Weight < fj.Weight
+
+}
 
 // GetBySuffix gets a output format given as suffix, e.g. "html".
 // It will return false if no format could be found, or if the suffix given
@@ -330,17 +361,20 @@ func decode(mediaTypes media.Types, input, output interface{}) error {
 	return decoder.Decode(input)
 }
 
-func (formats Format) BaseFilename() string {
-	return formats.BaseName + formats.MediaType.FullSuffix()
+// BaseFilename returns the base filename of f including an extension (ie.
+// "index.xml").
+func (f Format) BaseFilename() string {
+	return f.BaseName + f.MediaType.FullSuffix()
 }
 
-func (formats Format) MarshalJSON() ([]byte, error) {
+// MarshalJSON returns the JSON encoding of f.
+func (f Format) MarshalJSON() ([]byte, error) {
 	type Alias Format
 	return json.Marshal(&struct {
 		MediaType string
 		Alias
 	}{
-		MediaType: formats.MediaType.String(),
-		Alias:     (Alias)(formats),
+		MediaType: f.MediaType.String(),
+		Alias:     (Alias)(f),
 	})
 }
