@@ -1,4 +1,4 @@
-// Copyright 2017-present The Hugo Authors. All rights reserved.
+// Copyright 2019 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -73,6 +73,7 @@ type Config struct {
 	Indices IndexConfigs
 }
 
+// Add adds a given index.
 func (c *Config) Add(index IndexConfig) {
 	if c.ToLower {
 		index.ToLower = true
@@ -105,11 +106,15 @@ type IndexConfig struct {
 
 // Document is the interface an indexable document in Hugo must fulfill.
 type Document interface {
-	// SearchKeywords returns a list of keywords for the given index config.
-	SearchKeywords(cfg IndexConfig) ([]Keyword, error)
+	// RelatedKeywords returns a list of keywords for the given index config.
+	RelatedKeywords(cfg IndexConfig) ([]Keyword, error)
 
 	// When this document was or will be published.
-	PubDate() time.Time
+	PublishDate() time.Time
+
+	// Name is used as an tiebreaker if both Weight and PublishDate are
+	// the same.
+	Name() string
 }
 
 // InvertedIndex holds an inverted index, also sometimes named posting list, which
@@ -163,7 +168,7 @@ func (idx *InvertedIndex) Add(docs ...Document) error {
 
 		for _, doc := range docs {
 			var words []Keyword
-			words, err = doc.SearchKeywords(config)
+			words, err = doc.RelatedKeywords(config)
 			if err != nil {
 				continue
 			}
@@ -210,7 +215,10 @@ func (r ranks) Len() int      { return len(r) }
 func (r ranks) Swap(i, j int) { r[i], r[j] = r[j], r[i] }
 func (r ranks) Less(i, j int) bool {
 	if r[i].Weight == r[j].Weight {
-		return r[i].Doc.PubDate().After(r[j].Doc.PubDate())
+		if r[i].Doc.PublishDate() == r[j].Doc.PublishDate() {
+			return r[i].Doc.Name() < r[j].Doc.Name()
+		}
+		return r[i].Doc.PublishDate().After(r[j].Doc.PublishDate())
 	}
 	return r[i].Weight > r[j].Weight
 }
@@ -240,7 +248,7 @@ func (idx *InvertedIndex) SearchDoc(doc Document, indices ...string) ([]Document
 	}
 
 	for _, cfg := range configs {
-		keywords, err := doc.SearchKeywords(cfg)
+		keywords, err := doc.RelatedKeywords(cfg)
 		if err != nil {
 			return nil, err
 		}
@@ -249,9 +257,10 @@ func (idx *InvertedIndex) SearchDoc(doc Document, indices ...string) ([]Document
 
 	}
 
-	return idx.searchDate(doc.PubDate(), q...)
+	return idx.searchDate(doc.PublishDate(), q...)
 }
 
+// ToKeywords returns a Keyword slice of the given input.
 func (cfg IndexConfig) ToKeywords(v interface{}) ([]Keyword, error) {
 	var (
 		keywords []Keyword
@@ -279,7 +288,7 @@ func (cfg IndexConfig) ToKeywords(v interface{}) ([]Keyword, error) {
 	case nil:
 		return keywords, nil
 	default:
-		return keywords, fmt.Errorf("indexing currently not supported for for index %q and type %T", cfg.Name, vv)
+		return keywords, fmt.Errorf("indexing currently not supported for index %q and type %T", cfg.Name, vv)
 	}
 
 	return keywords, nil
@@ -342,7 +351,7 @@ func (idx *InvertedIndex) searchDate(upperDate time.Time, query ...queryElement)
 				for _, doc := range docs {
 					if applyDateFilter {
 						// Exclude newer than the limit given
-						if doc.PubDate().After(upperDate) {
+						if doc.PublishDate().After(upperDate) {
 							continue
 						}
 					}
