@@ -255,7 +255,7 @@ func getGitInfosBefore(ref, tag, repo, repoPath string, remote bool) (gitInfos,
 }
 
 // Ignore autogenerated commits etc. in change log. This is a regexp.
-const ignoredCommits = "releaser?:|snapcraft:|Merge commit|Squashed|Revert"
+const ignoredCommits = "releaser?:|snapcraft:|Merge commit|Squashed"
 
 func gitLogBefore(ref, tag, repoPath string) (string, error) {
 	var prevTag string
