@@ -13,6 +13,8 @@
 
 package page
 
+import "strings"
+
 const (
 	KindPage = "page"
 
@@ -23,3 +25,16 @@ const (
 	KindTaxonomy     = "taxonomy"
 	KindTaxonomyTerm = "taxonomyTerm"
 )
+
+var kindMap = map[string]string{
+	strings.ToLower(KindPage):         KindPage,
+	strings.ToLower(KindHome):         KindHome,
+	strings.ToLower(KindSection):      KindSection,
+	strings.ToLower(KindTaxonomy):     KindTaxonomy,
+	strings.ToLower(KindTaxonomyTerm): KindTaxonomyTerm,
+}
+
+// GetKind gets the page kind given a string, empty if not found.
+func GetKind(s string) string {
+	return kindMap[strings.ToLower(s)]
+}
