@@ -32,16 +32,21 @@ var testdataPermalinks = []struct {
 	{":title", true, "spf13-vim-3.0-release-and-new-website"},
 	{"/:year-:month-:title", true, "/2012-04-spf13-vim-3.0-release-and-new-website"},
 	{"/:year/:yearday/:month/:monthname/:day/:weekday/:weekdayname/", true, "/2012/97/04/April/06/5/Friday/"}, // Dates
-	{"/:section/", true, "/blue/"},                                // Section
-	{"/:title/", true, "/spf13-vim-3.0-release-and-new-website/"}, // Title
-	{"/:slug/", true, "/the-slug/"},                               // Slug
-	{"/:filename/", true, "/test-page/"},                          // Filename
+	{"/:section/", true, "/blue/"},                                  // Section
+	{"/:title/", true, "/spf13-vim-3.0-release-and-new-website/"},   // Title
+	{"/:slug/", true, "/the-slug/"},                                 // Slug
+	{"/:filename/", true, "/test-page/"},                            // Filename
+	{"/:06-:1-:2-:Monday", true, "/12-4-6-Friday"},                  // Dates with Go formatting
+	{"/:2006_01_02_15_04_05.000", true, "/2012_04_06_03_01_59.000"}, // Complicated custom date format
 	// TODO(moorereason): need test scaffolding for this.
 	//{"/:sections/", false, "/blue/"},                              // Sections
 
 	// Failures
 	{"/blog/:fred", false, ""},
 	{"/:year//:title", false, ""},
+	{"/:TITLE", false, ""},      // case is not normalized
+	{"/:2017", false, ""},       // invalid date format
+	{"/:2006-01-02", false, ""}, // valid date format but invalid attribute name
 }
 
 func TestPermalinkExpansion(t *testing.T) {
@@ -51,7 +56,7 @@ func TestPermalinkExpansion(t *testing.T) {
 
 	page := newTestPageWithFile("/test-page/index.md")
 	page.title = "Spf13 Vim 3.0 Release and new website"
-	d, _ := time.Parse("2006-01-02", "2012-04-06")
+	d, _ := time.Parse("2006-01-02 15:04:05", "2012-04-06 03:01:59")
 	page.date = d
 	page.section = "blue"
 	page.slug = "The Slug"
