@@ -1,4 +1,4 @@
-// Copyright 2017-present The Hugo Authors. All rights reserved.
+// Copyright 2019 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -14,14 +14,15 @@
 package source
 
 import (
-	"fmt"
-	"io"
-	"os"
 	"path/filepath"
 	"strings"
 	"sync"
 
-	"github.com/spf13/afero"
+	"github.com/gohugoio/hugo/hugofs/files"
+
+	"github.com/pkg/errors"
+
+	"github.com/gohugoio/hugo/common/hugio"
 
 	"github.com/gohugoio/hugo/hugofs"
 
@@ -30,38 +31,52 @@ import (
 
 // fileInfo implements the File interface.
 var (
-	_ File         = (*FileInfo)(nil)
-	_ ReadableFile = (*FileInfo)(nil)
+	_ File = (*FileInfo)(nil)
 )
 
+// File represents a source file.
+// This is a temporary construct until we resolve page.Page conflicts.
+// TODO(bep) remove this construct once we have resolved page deprecations
 type File interface {
+	fileOverlap
+	FileWithoutOverlap
+}
 
-	// Filename gets the full path and filename to the file.
-	Filename() string
-
+// Temporary to solve duplicate/deprecated names in page.Page
+type fileOverlap interface {
 	// Path gets the relative path including file name and extension.
 	// The directory is relative to the content root.
 	Path() string
 
+	// Section is first directory below the content root.
+	// For page bundles in root, the Section will be empty.
+	Section() string
+
+	// Lang is the language code for this page. It will be the
+	// same as the site's language code.
+	Lang() string
+
+	IsZero() bool
+}
+
+type FileWithoutOverlap interface {
+
+	// Filename gets the full path and filename to the file.
+	Filename() string
+
 	// Dir gets the name of the directory that contains this file.
 	// The directory is relative to the content root.
 	Dir() string
 
 	// Extension gets the file extension, i.e "myblogpost.md" will return "md".
 	Extension() string
+
 	// Ext is an alias for Extension.
 	Ext() string // Hmm... Deprecate Extension
 
-	// Lang for this page, if `Multilingual` is enabled on your site.
-	Lang() string
-
 	// LogicalName is filename and extension of the file.
 	LogicalName() string
 
-	// Section is first directory below the content root.
-	// For page bundles in root, the Section will be empty.
-	Section() string
-
 	// BaseFileName is a filename without extension.
 	BaseFileName() string
 
@@ -69,21 +84,18 @@ type File interface {
 	// not even the optional language extension part.
 	TranslationBaseName() string
 
+	// ContentBaseName is a either TranslationBaseName or name of containing folder
+	// if file is a leaf bundle.
+	ContentBaseName() string
+
 	// UniqueID is the MD5 hash of the file's path and is for most practical applications,
 	// Hugo content files being one of them, considered to be unique.
 	UniqueID() string
 
-	FileInfo() os.FileInfo
-
-	String() string
-}
-
-// A ReadableFile is a File that is readable.
-type ReadableFile interface {
-	File
-	Open() (io.ReadCloser, error)
+	FileInfo() hugofs.FileMetaInfo
 }
 
+// FileInfo describes a source file.
 type FileInfo struct {
 
 	// Absolute filename to the file on disk.
@@ -91,7 +103,7 @@ type FileInfo struct {
 
 	sp *SourceSpec
 
-	fi os.FileInfo
+	fi hugofs.FileMetaInfo
 
 	// Derived from filename
 	ext  string // Extension without any "."
@@ -104,6 +116,7 @@ type FileInfo struct {
 	relPath             string
 	baseName            string
 	translationBaseName string
+	contentBaseName     string
 	section             string
 	isLeafBundle        bool
 
@@ -112,31 +125,71 @@ type FileInfo struct {
 	lazyInit sync.Once
 }
 
-func (fi *FileInfo) Filename() string            { return fi.filename }
-func (fi *FileInfo) Path() string                { return fi.relPath }
-func (fi *FileInfo) Dir() string                 { return fi.relDir }
-func (fi *FileInfo) Extension() string           { return fi.Ext() }
-func (fi *FileInfo) Ext() string                 { return fi.ext }
-func (fi *FileInfo) Lang() string                { return fi.lang }
-func (fi *FileInfo) LogicalName() string         { return fi.name }
-func (fi *FileInfo) BaseFileName() string        { return fi.baseName }
+// Filename returns a file's absolute path and filename on disk.
+func (fi *FileInfo) Filename() string { return fi.filename }
+
+// Path gets the relative path including file name and extension.  The directory
+// is relative to the content root.
+func (fi *FileInfo) Path() string { return fi.relPath }
+
+// Dir gets the name of the directory that contains this file.  The directory is
+// relative to the content root.
+func (fi *FileInfo) Dir() string { return fi.relDir }
+
+// Extension is an alias to Ext().
+func (fi *FileInfo) Extension() string { return fi.Ext() }
+
+// Ext returns a file's extension without the leading period (ie. "md").
+func (fi *FileInfo) Ext() string { return fi.ext }
+
+// Lang returns a file's language (ie. "sv").
+func (fi *FileInfo) Lang() string { return fi.lang }
+
+// LogicalName returns a file's name and extension (ie. "page.sv.md").
+func (fi *FileInfo) LogicalName() string { return fi.name }
+
+// BaseFileName returns a file's name without extension (ie. "page.sv").
+func (fi *FileInfo) BaseFileName() string { return fi.baseName }
+
+// TranslationBaseName returns a file's translation base name without the
+// language segement (ie. "page").
 func (fi *FileInfo) TranslationBaseName() string { return fi.translationBaseName }
 
+// ContentBaseName is a either TranslationBaseName or name of containing folder
+// if file is a leaf bundle.
+func (fi *FileInfo) ContentBaseName() string {
+	fi.init()
+	return fi.contentBaseName
+}
+
+// Section returns a file's section.
 func (fi *FileInfo) Section() string {
 	fi.init()
 	return fi.section
 }
 
+// UniqueID returns a file's unique, MD5 hash identifier.
 func (fi *FileInfo) UniqueID() string {
 	fi.init()
 	return fi.uniqueID
 }
-func (fi *FileInfo) FileInfo() os.FileInfo {
-	return fi.fi
-}
+
+// FileInfo returns a file's underlying os.FileInfo.
+func (fi *FileInfo) FileInfo() hugofs.FileMetaInfo { return fi.fi }
 
 func (fi *FileInfo) String() string { return fi.BaseFileName() }
 
+// Open implements ReadableFile.
+func (fi *FileInfo) Open() (hugio.ReadSeekCloser, error) {
+	f, err := fi.fi.Meta().Open()
+
+	return f, err
+}
+
+func (fi *FileInfo) IsZero() bool {
+	return fi == nil
+}
+
 // We create a lot of these FileInfo objects, but there are parts of it used only
 // in some cases that is slightly expensive to construct.
 func (fi *FileInfo) init() {
@@ -147,45 +200,67 @@ func (fi *FileInfo) init() {
 		if (!fi.isLeafBundle && len(parts) == 1) || len(parts) > 1 {
 			section = parts[0]
 		}
-
 		fi.section = section
 
-		fi.uniqueID = helpers.MD5String(filepath.ToSlash(fi.relPath))
+		if fi.isLeafBundle && len(parts) > 0 {
+			fi.contentBaseName = parts[len(parts)-1]
+		} else {
+			fi.contentBaseName = fi.translationBaseName
+		}
 
+		fi.uniqueID = helpers.MD5String(filepath.ToSlash(fi.relPath))
 	})
 }
 
-func (sp *SourceSpec) NewFileInfo(baseDir, filename string, isLeafBundle bool, fi os.FileInfo) *FileInfo {
+// NewTestFile creates a partially filled File used in unit tests.
+// TODO(bep) improve this package
+func NewTestFile(filename string) *FileInfo {
+	base := filepath.Base(filepath.Dir(filename))
+	return &FileInfo{
+		filename:            filename,
+		translationBaseName: base,
+	}
+}
 
-	var lang, translationBaseName, relPath string
-
-	if fp, ok := fi.(hugofs.FilePather); ok {
-		filename = fp.Filename()
-		baseDir = fp.BaseDir()
-		relPath = fp.Path()
+func (sp *SourceSpec) NewFileInfoFrom(path, filename string) (*FileInfo, error) {
+	meta := hugofs.FileMeta{
+		"filename": filename,
+		"path":     path,
 	}
 
-	if fl, ok := fi.(hugofs.LanguageAnnouncer); ok {
-		lang = fl.Lang()
-		translationBaseName = fl.TranslationBaseName()
-	}
+	return sp.NewFileInfo(hugofs.NewFileMetaInfo(nil, meta))
+}
 
-	dir, name := filepath.Split(filename)
-	if !strings.HasSuffix(dir, helpers.FilePathSeparator) {
-		dir = dir + helpers.FilePathSeparator
-	}
+func (sp *SourceSpec) NewFileInfo(fi hugofs.FileMetaInfo) (*FileInfo, error) {
 
-	baseDir = strings.TrimSuffix(baseDir, helpers.FilePathSeparator)
+	m := fi.Meta()
 
-	relDir := ""
-	if dir != baseDir {
-		relDir = strings.TrimPrefix(dir, baseDir)
-	}
-
-	relDir = strings.TrimPrefix(relDir, helpers.FilePathSeparator)
+	filename := m.Filename()
+	relPath := m.Path()
+	isLeafBundle := m.Classifier() == files.ContentClassLeaf
 
 	if relPath == "" {
-		relPath = filepath.Join(relDir, name)
+		return nil, errors.Errorf("no Path provided by %v (%T)", m, m.Fs())
+	}
+
+	if filename == "" {
+		return nil, errors.Errorf("no Filename provided by %v (%T)", m, m.Fs())
+	}
+
+	relDir := filepath.Dir(relPath)
+	if relDir == "." {
+		relDir = ""
+	}
+	if !strings.HasSuffix(relDir, helpers.FilePathSeparator) {
+		relDir = relDir + helpers.FilePathSeparator
+	}
+
+	lang := m.Lang()
+	translationBaseName := m.GetString("translationBaseName")
+
+	dir, name := filepath.Split(relPath)
+	if !strings.HasSuffix(dir, helpers.FilePathSeparator) {
+		dir = dir + helpers.FilePathSeparator
 	}
 
 	ext := strings.ToLower(strings.TrimPrefix(filepath.Ext(name), "."))
@@ -206,41 +281,14 @@ func (sp *SourceSpec) NewFileInfo(baseDir, filename string, isLeafBundle bool, f
 		lang:                lang,
 		ext:                 ext,
 		dir:                 dir,
-		relDir:              relDir,
-		relPath:             relPath,
+		relDir:              relDir,  // Dir()
+		relPath:             relPath, // Path()
 		name:                name,
-		baseName:            baseName,
+		baseName:            baseName, // BaseFileName()
 		translationBaseName: translationBaseName,
 		isLeafBundle:        isLeafBundle,
 	}
 
-	return f
+	return f, nil
 
 }
-
-// Open implements ReadableFile.
-func (fi *FileInfo) Open() (io.ReadCloser, error) {
-	f, err := fi.sp.SourceFs.Open(fi.Filename())
-	return f, err
-}
-
-func printFs(fs afero.Fs, path string, w io.Writer) {
-	if fs == nil {
-		return
-	}
-	afero.Walk(fs, path, func(path string, info os.FileInfo, err error) error {
-
-		if info != nil && !info.IsDir() {
-
-			s := path
-			if lang, ok := info.(hugofs.LanguageAnnouncer); ok {
-				s = s + "\t" + lang.Lang()
-			}
-			if fp, ok := info.(hugofs.FilePather); ok {
-				s = s + "\t" + fp.Filename()
-			}
-			fmt.Fprintln(w, "    ", s)
-		}
-		return nil
-	})
-}
