@@ -17,7 +17,9 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
+	"runtime"
 
+	"github.com/gohugoio/hugo/langs"
 	"github.com/spf13/afero"
 
 	"github.com/gohugoio/hugo/helpers"
@@ -29,7 +31,7 @@ import (
 type SourceSpec struct {
 	*helpers.PathSpec
 
-	Fs afero.Fs
+	SourceFs afero.Fs
 
 	// This is set if the ignoreFiles config is set.
 	ignoreFilesRe []*regexp.Regexp
@@ -52,7 +54,7 @@ func NewSourceSpec(ps *helpers.PathSpec, fs afero.Fs) *SourceSpec {
 	}
 
 	if len(languages) == 0 {
-		l := helpers.NewDefaultLanguage(cfg)
+		l := langs.NewDefaultLanguage(cfg)
 		languages[l.Lang] = l
 		defaultLang = l.Lang
 	}
@@ -71,12 +73,14 @@ func NewSourceSpec(ps *helpers.PathSpec, fs afero.Fs) *SourceSpec {
 		}
 	}
 
-	return &SourceSpec{ignoreFilesRe: regexps, PathSpec: ps, Fs: fs, Languages: languages, DefaultContentLanguage: defaultLang, DisabledLanguages: disabledLangsSet}
+	return &SourceSpec{ignoreFilesRe: regexps, PathSpec: ps, SourceFs: fs, Languages: languages, DefaultContentLanguage: defaultLang, DisabledLanguages: disabledLangsSet}
+
 }
 
+// IgnoreFile returns whether a given file should be ignored.
 func (s *SourceSpec) IgnoreFile(filename string) bool {
 	if filename == "" {
-		if _, ok := s.Fs.(*afero.OsFs); ok {
+		if _, ok := s.SourceFs.(*afero.OsFs); ok {
 			return true
 		}
 		return false
@@ -104,11 +108,25 @@ func (s *SourceSpec) IgnoreFile(filename string) bool {
 		}
 	}
 
+	if runtime.GOOS == "windows" {
+		// Also check the forward slash variant if different.
+		unixFilename := filepath.ToSlash(filename)
+		if unixFilename != filename {
+			for _, re := range s.ignoreFilesRe {
+				if re.MatchString(unixFilename) {
+					return true
+				}
+			}
+		}
+	}
+
 	return false
 }
 
+// IsRegularSourceFile returns whether filename represents a regular file in the
+// source filesystem.
 func (s *SourceSpec) IsRegularSourceFile(filename string) (bool, error) {
-	fi, err := helpers.LstatIfPossible(s.Fs, filename)
+	fi, err := helpers.LstatIfPossible(s.SourceFs, filename)
 	if err != nil {
 		return false, err
 	}
@@ -119,7 +137,11 @@ func (s *SourceSpec) IsRegularSourceFile(filename string) (bool, error) {
 
 	if fi.Mode()&os.ModeSymlink == os.ModeSymlink {
 		link, err := filepath.EvalSymlinks(filename)
-		fi, err = helpers.LstatIfPossible(s.Fs, link)
+		if err != nil {
+			return false, err
+		}
+
+		fi, err = helpers.LstatIfPossible(s.SourceFs, link)
 		if err != nil {
 			return false, err
 		}
