@@ -14,10 +14,12 @@
 package collections
 
 import (
-	"errors"
 	"fmt"
 	"reflect"
 	"time"
+
+	"github.com/mitchellh/hashstructure"
+	"github.com/pkg/errors"
 )
 
 var (
@@ -37,7 +39,71 @@ func numberToFloat(v reflect.Value) (float64, error) {
 	case kind == reflect.Interface:
 		return numberToFloat(v.Elem())
 	default:
-		return 0, fmt.Errorf("Invalid kind %s in numberToFloat", kind)
+		return 0, fmt.Errorf("invalid kind %s in numberToFloat", kind)
+	}
+}
+
+// normalizes different numeric types if isNumber
+// or get the hash values if not Comparable (such as map or struct)
+// to make them comparable
+func normalize(v reflect.Value) interface{} {
+	k := v.Kind()
+
+	switch {
+	case !v.Type().Comparable():
+		h, err := hashstructure.Hash(v.Interface(), nil)
+		if err != nil {
+			panic(err)
+		}
+		return h
+	case isNumber(k):
+		f, err := numberToFloat(v)
+		if err == nil {
+			return f
+		}
+	}
+	return v.Interface()
+}
+
+// collects identities from the slices in seqs into a set. Numeric values are normalized,
+// pointers unwrapped.
+func collectIdentities(seqs ...interface{}) (map[interface{}]bool, error) {
+	seen := make(map[interface{}]bool)
+	for _, seq := range seqs {
+		v := reflect.ValueOf(seq)
+		switch v.Kind() {
+		case reflect.Array, reflect.Slice:
+			for i := 0; i < v.Len(); i++ {
+				ev, _ := indirectInterface(v.Index(i))
+
+				if !ev.Type().Comparable() {
+					return nil, errors.New("elements must be comparable")
+				}
+
+				seen[normalize(ev)] = true
+			}
+		default:
+			return nil, fmt.Errorf("arguments must be slices or arrays")
+		}
+	}
+
+	return seen, nil
+}
+
+// We have some different numeric and string types that we try to behave like
+// they were the same.
+func convertValue(v reflect.Value, to reflect.Type) (reflect.Value, error) {
+	if v.Type().AssignableTo(to) {
+		return v, nil
+	}
+	switch kind := to.Kind(); {
+	case kind == reflect.String:
+		s, err := toString(v)
+		return reflect.ValueOf(s), err
+	case isNumber(kind):
+		return convertNumber(v, kind)
+	default:
+		return reflect.Value{}, errors.Errorf("%s is not assignable to %s", v.Type(), to)
 	}
 }
 
@@ -102,6 +168,23 @@ func convertNumber(v reflect.Value, to reflect.Kind) (reflect.Value, error) {
 
 }
 
+func newSliceElement(items interface{}) interface{} {
+	tp := reflect.TypeOf(items)
+	if tp == nil {
+		return nil
+	}
+	switch tp.Kind() {
+	case reflect.Array, reflect.Slice:
+		tp = tp.Elem()
+		if tp.Kind() == reflect.Ptr {
+			tp = tp.Elem()
+		}
+
+		return reflect.New(tp).Interface()
+	}
+	return nil
+}
+
 func isNumber(kind reflect.Kind) bool {
 	return isInt(kind) || isUint(kind) || isFloat(kind)
 }
