@@ -11,6 +11,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// Package compare provides template functions for comparing values.
 package compare
 
 import (
@@ -20,15 +21,19 @@ import (
 	"time"
 
 	"github.com/gohugoio/hugo/compare"
+
+	"github.com/gohugoio/hugo/common/types"
 )
 
 // New returns a new instance of the compare-namespaced template functions.
-func New() *Namespace {
-	return &Namespace{}
+func New(caseInsensitive bool) *Namespace {
+	return &Namespace{caseInsensitive: caseInsensitive}
 }
 
 // Namespace provides template functions for the "compare" namespace.
 type Namespace struct {
+	// Enable to do case insensitive string compares.
+	caseInsensitive bool
 }
 
 // Default checks whether a given value is set and returns a default value if it
@@ -85,17 +90,19 @@ func (*Namespace) Default(dflt interface{}, given ...interface{}) (interface{},
 	return dflt, nil
 }
 
-// Eq returns the boolean truth of arg1 == arg2.
-func (*Namespace) Eq(x, y interface{}) bool {
-
-	// hugolib.Page implements compare.Eqer to make Page and PageOutput comparable.
-	if e1, ok := x.(compare.Eqer); ok {
-		if e2, ok := y.(compare.Eqer); ok {
-			return e1.Eq(e2)
-		}
+// Eq returns the boolean truth of arg1 == arg2 || arg1 == arg3 || arg1 == arg4.
+func (n *Namespace) Eq(first interface{}, others ...interface{}) bool {
+	if n.caseInsensitive {
+		panic("caseInsensitive not implemented for Eq")
+	}
+	if len(others) == 0 {
+		panic("missing arguments for comparison")
 	}
 
 	normalize := func(v interface{}) interface{} {
+		if types.IsNil(v) {
+			return nil
+		}
 		vv := reflect.ValueOf(v)
 		switch vv.Kind() {
 		case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
@@ -104,42 +111,90 @@ func (*Namespace) Eq(x, y interface{}) bool {
 			return vv.Float()
 		case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64:
 			return vv.Uint()
+		case reflect.String:
+			return vv.String()
 		default:
 			return v
 		}
 	}
-	x = normalize(x)
-	y = normalize(y)
-	return reflect.DeepEqual(x, y)
+
+	normFirst := normalize(first)
+	for _, other := range others {
+		if e, ok := first.(compare.Eqer); ok {
+			if e.Eq(other) {
+				return true
+			}
+			continue
+		}
+
+		if e, ok := other.(compare.Eqer); ok {
+			if e.Eq(first) {
+				return true
+			}
+			continue
+		}
+
+		other = normalize(other)
+		if reflect.DeepEqual(normFirst, other) {
+			return true
+		}
+	}
+
+	return false
 }
 
-// Ne returns the boolean truth of arg1 != arg2.
-func (n *Namespace) Ne(x, y interface{}) bool {
-	return !n.Eq(x, y)
+// Ne returns the boolean truth of arg1 != arg2 && arg1 != arg3 && arg1 != arg4.
+func (n *Namespace) Ne(first interface{}, others ...interface{}) bool {
+	for _, other := range others {
+		if n.Eq(first, other) {
+			return false
+		}
+	}
+	return true
 }
 
-// Ge returns the boolean truth of arg1 >= arg2.
-func (n *Namespace) Ge(a, b interface{}) bool {
-	left, right := n.compareGetFloat(a, b)
-	return left >= right
+// Ge returns the boolean truth of arg1 >= arg2 && arg1 >= arg3 && arg1 >= arg4.
+func (n *Namespace) Ge(first interface{}, others ...interface{}) bool {
+	for _, other := range others {
+		left, right := n.compareGet(first, other)
+		if !(left >= right) {
+			return false
+		}
+	}
+	return true
 }
 
-// Gt returns the boolean truth of arg1 > arg2.
-func (n *Namespace) Gt(a, b interface{}) bool {
-	left, right := n.compareGetFloat(a, b)
-	return left > right
+// Gt returns the boolean truth of arg1 > arg2 && arg1 > arg3 && arg1 > arg4.
+func (n *Namespace) Gt(first interface{}, others ...interface{}) bool {
+	for _, other := range others {
+		left, right := n.compareGet(first, other)
+		if !(left > right) {
+			return false
+		}
+	}
+	return true
 }
 
-// Le returns the boolean truth of arg1 <= arg2.
-func (n *Namespace) Le(a, b interface{}) bool {
-	left, right := n.compareGetFloat(a, b)
-	return left <= right
+// Le returns the boolean truth of arg1 <= arg2 && arg1 <= arg3 && arg1 <= arg4.
+func (n *Namespace) Le(first interface{}, others ...interface{}) bool {
+	for _, other := range others {
+		left, right := n.compareGet(first, other)
+		if !(left <= right) {
+			return false
+		}
+	}
+	return true
 }
 
-// Lt returns the boolean truth of arg1 < arg2.
-func (n *Namespace) Lt(a, b interface{}) bool {
-	left, right := n.compareGetFloat(a, b)
-	return left < right
+// Lt returns the boolean truth of arg1 < arg2 && arg1 < arg3 && arg1 < arg4.
+func (n *Namespace) Lt(first interface{}, others ...interface{}) bool {
+	for _, other := range others {
+		left, right := n.compareGet(first, other)
+		if !(left < right) {
+			return false
+		}
+	}
+	return true
 }
 
 // Conditional can be used as a ternary operator.
@@ -151,7 +206,29 @@ func (n *Namespace) Conditional(condition bool, a, b interface{}) interface{} {
 	return b
 }
 
-func (*Namespace) compareGetFloat(a interface{}, b interface{}) (float64, float64) {
+func (ns *Namespace) compareGet(a interface{}, b interface{}) (float64, float64) {
+	if ac, ok := a.(compare.Comparer); ok {
+		c := ac.Compare(b)
+		if c < 0 {
+			return 1, 0
+		} else if c == 0 {
+			return 0, 0
+		} else {
+			return 0, 1
+		}
+	}
+
+	if bc, ok := b.(compare.Comparer); ok {
+		c := bc.Compare(a)
+		if c < 0 {
+			return 0, 1
+		} else if c == 0 {
+			return 0, 0
+		} else {
+			return 1, 0
+		}
+	}
+
 	var left, right float64
 	var leftStr, rightStr *string
 	av := reflect.ValueOf(a)
@@ -200,6 +277,17 @@ func (*Namespace) compareGetFloat(a interface{}, b interface{}) (float64, float6
 		}
 	}
 
+	if ns.caseInsensitive && leftStr != nil && rightStr != nil {
+		c := compare.Strings(*leftStr, *rightStr)
+		if c < 0 {
+			return 0, 1
+		} else if c > 0 {
+			return 1, 0
+		} else {
+			return 0, 0
+		}
+	}
+
 	switch {
 	case leftStr == nil || rightStr == nil:
 	case *leftStr < *rightStr:
