@@ -11,6 +11,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// Package fmt provides template functions for formatting strings.
 package fmt
 
 import (
@@ -22,12 +23,16 @@ import (
 
 // New returns a new instance of the fmt-namespaced template functions.
 func New(d *deps.Deps) *Namespace {
-	return &Namespace{helpers.NewDistinctLogger(d.Log.ERROR)}
+	return &Namespace{
+		errorLogger: helpers.NewDistinctLogger(d.Log.ERROR),
+		warnLogger:  helpers.NewDistinctLogger(d.Log.WARN),
+	}
 }
 
 // Namespace provides template functions for the "fmt" namespace.
 type Namespace struct {
 	errorLogger *helpers.DistinctLogger
+	warnLogger  *helpers.DistinctLogger
 }
 
 // Print returns string representation of the passed arguments.
@@ -46,9 +51,16 @@ func (ns *Namespace) Println(a ...interface{}) string {
 	return _fmt.Sprintln(a...)
 }
 
-// Errorf formats according to a format specifier and returns the string as a
-// value that satisfies error.
+// Errorf formats according to a format specifier and logs an ERROR.
+// It returns an empty string.
 func (ns *Namespace) Errorf(format string, a ...interface{}) string {
 	ns.errorLogger.Printf(format, a...)
-	return _fmt.Sprintf(format, a...)
+	return ""
+}
+
+// Warnf formats according to a format specifier and logs a WARNING.
+// It returns an empty string.
+func (ns *Namespace) Warnf(format string, a ...interface{}) string {
+	ns.warnLogger.Printf(format, a...)
+	return ""
 }
