@@ -11,6 +11,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// Package math provides template functions for mathmatical operations.
 package math
 
 import (
@@ -71,17 +72,29 @@ func (ns *Namespace) Log(a interface{}) (float64, error) {
 	return math.Log(af), nil
 }
 
+// Sqrt returns the square root of a number.
+// NOTE: will return for NaN for negative values of a
+func (ns *Namespace) Sqrt(a interface{}) (float64, error) {
+	af, err := cast.ToFloat64E(a)
+
+	if err != nil {
+		return 0, errors.New("Sqrt operator can't be used with non integer or float value")
+	}
+
+	return math.Sqrt(af), nil
+}
+
 // Mod returns a % b.
 func (ns *Namespace) Mod(a, b interface{}) (int64, error) {
 	ai, erra := cast.ToInt64E(a)
 	bi, errb := cast.ToInt64E(b)
 
 	if erra != nil || errb != nil {
-		return 0, errors.New("Modulo operator can't be used with non integer value")
+		return 0, errors.New("modulo operator can't be used with non integer value")
 	}
 
 	if bi == 0 {
-		return 0, errors.New("The number can't be divided by zero at modulo operation")
+		return 0, errors.New("the number can't be divided by zero at modulo operation")
 	}
 
 	return ai % bi, nil
